﻿/*
 * mTools - © EUROPEAN COMMISSION, 2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous
 * la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou 
 * implicites.
 * Consultez la Licence pour les autorisations et les restrictions 
 * linguistiques spécifiques relevant de la Licence.
 *
 * Auteur de l'oeuvre: Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using MetaNull.Console;

namespace MetaNull.Diagnostics
{
	/// <summary>
	/// Helper class keeping traces of the memory allocation from developer defined steps in the code
	/// </summary>
	public class MemoryTrace
	{
		/// <summary>
		/// Time of the snapshot
		/// </summary>
		public DateTime time;
		/// <summary>
		/// Title of the snapshot
		/// </summary>
		public String label;
		/// <summary>
		/// Total memory allocated
		/// </summary>
		public long memory;

		private const int padSize = 10;

		/// <summary>
		/// Create a new trace snapshot
		/// </summary>
		/// <param name="label">Title of the snapshot</param>
		public MemoryTrace(string label)
		{
			this.time = DateTime.Now;
			this.label = label;
			GC.Collect();
			GC.WaitForPendingFinalizers();
			this.memory = GC.GetTotalMemory(true);
		}
		/// <summary>
		/// Dump memory allocation between two specific steps of the program
		/// </summary>
		/// <param name="prevStep">Previous step</param>
		/// <param name="curStep">Current Step</param>
		public static void DumpTraces(MemoryTrace prevStep, MemoryTrace curStep)
		{
			DumpTraces(new MemoryTrace[] { prevStep, curStep });
		}
		/// <summary>
		/// Dump memory allocation for multiple steps of the program
		/// </summary>
		/// <param name="traces">List of snapshots</param>
		public static void DumpTraces(MemoryTrace[] traces)
		{
			if (traces.Length == 0)
			{
				return;
			}
			MemoryTrace firstTrace = traces.First();
			MemoryTrace lastTrace = traces.Last();
			MemoryTrace prevTrace = null;
			foreach (MemoryTrace trace in traces)
			{
				if (null == prevTrace)
				{
					firstTrace = trace;

					SynchronizedConsole.WriteLine(
						firstTrace.time.ToString("hh:mm:ss.fffffff") + " " +
						Math.Round((firstTrace.memory) / 1024.0, 3).ToString() + "kb " +
						firstTrace.label, ConsoleColor.Green
					);
				}
				else
				{
					if (trace != lastTrace)
					{
						SynchronizedConsole.WriteLine(
							(trace.time - prevTrace.time).TotalMilliseconds.ToString().PadLeft(padSize) + "ms " +
							Math.Round((trace.memory - prevTrace.memory) / 1024.0, 3).ToString().PadLeft(padSize) + "kb " +
							trace.label, ConsoleColor.White
						);
					}
				}
				prevTrace = trace;
			}
			if (2 != traces.Length)
			{
				SynchronizedConsole.WriteLine(
					firstTrace.time.ToString("hh:mm:ss.fffffff") + " " +
					Math.Round((firstTrace.memory) / 1024.0, 3).ToString() + "kb ",
					ConsoleColor.Red
				);
			}
			SynchronizedConsole.WriteLine(
				(lastTrace.time - firstTrace.time).TotalMilliseconds.ToString().PadLeft(padSize) + "ms " +
				Math.Round((lastTrace.memory - firstTrace.memory) / 1024.0, 3).ToString().PadLeft(padSize) + "kb " +
				lastTrace.label,
				ConsoleColor.Red
			);
		}
	}
}
