﻿/*
 * mTools - © EUROPEAN COMMISSION, 2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous
 * la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou 
 * implicites.
 * Consultez la Licence pour les autorisations et les restrictions 
 * linguistiques spécifiques relevant de la Licence.
 *
 * Auteur de l'oeuvre: Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.IO;
using System.Threading;
using MetaNull.Threading;

namespace MetaNull.IO
{
	/// <summary>
	/// Instanciates a FileSystemWatcher object and implements a thread in order
	/// to filter out the events triggered by FileSystemWatcher
	/// </summary>
	/// <remarks>
	/// FileSystemWatcher can trigger multiple events when a file is created.
	/// FilesystemMonitor will only trigger one event when the file is created
	/// and closed by the originating process (If a large file is copied on the 
	/// filesystem, the event will be triggered after ALL the data has been 
	/// transferred)
	/// event after the creation of a fileOnly one event will be triggered at 
	/// </remarks>
	public class FilesystemMonitor : IDisposable
	{

		#region Events

		/// <summary>
		/// A file has been created and all the data has been copied into it
		/// </summary>
		/// <param name="FullPath">Full path of the new file</param>
		public delegate void FileCreatedHandler(string FullPath);
		/// <summary>
		/// A file has been deleted
		/// </summary>
		/// <param name="FullPath">Full path of the deleted file</param>
		public delegate void FileDeletedHandler(string FullPath);
		/// <summary>
		/// A file has been renamed
		/// </summary>
		/// <param name="OldFullPath">Full path of the old file</param>
		/// <param name="NewFullPath">Full path of the new file</param>
		public delegate void FileRenamedHandler(string OldFullPath, string NewFullPath);
		/// <summary>
		/// An error has occured
		/// </summary>
		/// <param name="Error">Error description</param>
		public delegate void ThreadErrorHandler(Exception Error);

		/// <summary>
		/// Triggered after the file has been created and all the data has been 
		/// copied into it
		/// </summary>
		public event FileCreatedHandler OnFileCreated;
		/// <summary>
		/// Triggered after a file has been deleted
		/// </summary>
		public event FileDeletedHandler OnFileDeleted;
		/// <summary>
		/// Triggered after a file has been renamed
		/// </summary>
		public event FileRenamedHandler OnFileRenamed;
		/// <summary>
		/// Triggered whenever an error occurs in the internal thread
		/// In which case you will probably have to dispose of the current
		/// instance and instanciate a new object
		/// </summary>
		public event ThreadErrorHandler OnThreadError;

		#endregion

		#region Class Internals

		private FileSystemWatcher _Watcher = null;
		private Thread _Monitor = null;
		private ManualResetEvent _MonitorStopCondition = null;
		private int _YieldTime = 10;
		private int _PollInterval = 500;
		private int _MonitorStopTimeout = 1000;
		private object _EventListLock = new object();
		private Dictionary<string, DateTime> _EventList = new Dictionary<string, DateTime>();

		/// <summary>
		/// Default Constructor
		/// </summary>
		public FilesystemMonitor()
		{
			_Watcher = new FileSystemWatcher();
			InitFSW();
		}
		/// <summary>
		/// Constructor with initialisation
		/// </summary>
		/// <param name="path">Path to monitor</param>
		public FilesystemMonitor(string path)
		{
			_Watcher = new FileSystemWatcher(path);
			InitFSW();
		}
		/// <summary>
		/// Constructor with initialisation
		/// </summary>
		/// <param name="path">Path to monitor</param>
		/// <param name="filter">Filename filter (I.e.:...*/txt; *.5</param>
		public FilesystemMonitor(string path, string filter)
		{
			_Watcher = new FileSystemWatcher(path, filter);
			InitFSW();
		}
		private void InitFSW()
		{
			_Watcher.NotifyFilter =
				NotifyFilters.LastWrite | NotifyFilters.LastAccess |
				NotifyFilters.FileName | NotifyFilters.DirectoryName |
				NotifyFilters.Size;
			_Watcher.Created += new FileSystemEventHandler(_Watcher_Created);
			_Watcher.Changed += new FileSystemEventHandler(_Watcher_Changed);
			_Watcher.Deleted += new FileSystemEventHandler(_Watcher_Deleted);
			_Watcher.Renamed += new RenamedEventHandler(_Watcher_Renamed);
			_MonitorStopCondition = new ManualResetEvent(false);
			ParameterizedThreadStart start = new ParameterizedThreadStart(Monitor);
			_Monitor = new Thread(start);
			_Monitor.Start(new ThreadParameter());
		}

		#endregion

		#region Data accessors

		/// <summary>
		/// Get the ath ubt
		/// </summary>
		public string Path
		{
			get { return _Watcher.Path; }
			set { _Watcher.Path = value; }
		}
		/// <summary>
		/// Get the filenamefilter
		/// </summary>
		public string Filter
		{
			get { return _Watcher.Filter; }
			set { _Watcher.Filter = value;  }
		}

		/// <summary>
		/// Get/Set the state of monitoring
		/// </summary>
		public bool MonitoringEnabled
		{
			get { return _Watcher.EnableRaisingEvents; }
			set { _Watcher.EnableRaisingEvents = value; }
		}

		/// <summary>
		/// Get/Set	a value indicating whether subdirectories within the specified path should be monitored
		/// </summary>
		public bool IncludeSubdirectories 
		{
			get { return _Watcher.IncludeSubdirectories; }
			set { _Watcher.IncludeSubdirectories = value; }
		}

		#endregion

		#region Static Helpers

		/// <summary>
		/// Checks if a given file is currently locked
		/// </summary>
		/// <remarks>
		/// Tries to open the file in ReadWrite mode, with exclusive access
		/// </remarks>
		/// <param name="FullPath">Path to the file</param>
		/// <returns></returns>
		public static bool FileLocked(string FullPath)
		{
			try
			{
				using (FileStream fs = File.Open(FullPath, FileMode.Open, FileAccess.ReadWrite, FileShare.None))
				{
					fs.Close();
					return false;
				}
			}
			catch (IOException)
			{
				return true;
			}
		}

		#endregion

		#region Monitor Thread and Events

		/// <summary>
		/// Thread that browses the received events and report the newly created files to the caller
		/// </summary>
		/// <param name="ThreadParam"></param>
 		private void Monitor(object ThreadParam)
		{
			try
			{
				ThreadParameter param = (ThreadParameter)ThreadParam;
				DateTime lastRefresh = new DateTime(0);
				while (false == _MonitorStopCondition.WaitOne(_YieldTime))
				{
					if (MonitoringEnabled)
					{
						if (0 == lastRefresh.Ticks || _PollInterval < (DateTime.Now - lastRefresh).TotalMilliseconds)
						{
							Dictionary<string, DateTime> Copy;
							lock (_EventListLock)
							{
								Copy = new Dictionary<string, DateTime>(_EventList);
							}
							foreach (var Pair in Copy)
							{
								if (_PollInterval < (DateTime.Now - Pair.Value).TotalMilliseconds)
								{
									if (!FileLocked(Pair.Key))
									{
										if (null != OnFileCreated)
										{
											OnFileCreated(Pair.Key);
										}
										lock (_EventListLock)
										{
											_EventList.Remove(Pair.Key);
										}
									}
								}
							}
							lastRefresh = DateTime.Now;
						}
					}
				}
			}
			catch (Exception e)
			{
				if (null != OnThreadError)
				{
					OnThreadError(e);
				}
			}
		}

		/// <summary>
		/// Catch new file events
		/// </summary>
		/// <seealso cref="Monitor"/>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void _Watcher_Created(object sender, FileSystemEventArgs e)
		{
			lock (_EventListLock)
			{
				if (_EventList.ContainsKey(e.FullPath))
				{
					_EventList[e.FullPath] = DateTime.Now;
				}
				else
				{
					_EventList.Add(e.FullPath, DateTime.Now);
				}
			}
		}
		/// <summary>
		/// Catch file change events
		/// </summary>
		/// <seealso cref="Monitor"/>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void _Watcher_Changed(object sender, FileSystemEventArgs e)
		{
			lock (_EventListLock)
			{
				if (_EventList.ContainsKey(e.FullPath))
				{
					_EventList[e.FullPath] = DateTime.Now;
				}
			}
		}

		/// <summary>
		/// Reports deleted files (unless they were newly created files that were not yet reported to the caller)
		/// </summary>
		/// <seealso cref="Monitor"/>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void _Watcher_Deleted(object sender, FileSystemEventArgs e)
		{
			lock (_EventListLock)
			{
				if (_EventList.ContainsKey(e.FullPath))
				{
					_EventList.Remove(e.FullPath);
				}
				else
				{
					if (null != OnFileDeleted)
					{
						OnFileDeleted(e.FullPath);
					}
				}
			}
		}

		/// <summary>
		/// Reports renamed files (unless they were newly created files that were not yet reported to the caller)
		/// </summary>
		/// <seealso cref="Monitor"/>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		void _Watcher_Renamed(object sender, RenamedEventArgs e)
		{
			lock (_EventListLock)
			{
				if (_EventList.ContainsKey(e.OldFullPath))
				{
					_EventList.Remove(e.OldFullPath);
					if (_EventList.ContainsKey(e.FullPath))
					{
						_EventList[e.FullPath] = DateTime.Now;
					}
					else
					{
						_EventList.Add(e.FullPath, DateTime.Now);
					}
				}
				else
				{
					if (null != OnFileRenamed)
					{
						OnFileRenamed(e.OldFullPath, e.FullPath);
					}
				}
			}
		}

		#endregion

		#region IDisposable Implementation

		/// <summary>
		/// Destructor
		/// </summary>
		~FilesystemMonitor()
		{
			Dispose(false);
		}
		private bool disposed = false;

		/// <summary>
		/// Dispose of the object
		/// </summary>
		/// <param name="disposing"></param>
		public void Dispose(bool disposing)
		{
			if (!disposed)
			{
				if (disposing)
				{
					MonitoringEnabled = false;
					_MonitorStopCondition.Set();
					_Monitor.Join( _MonitorStopTimeout );
					_Watcher.Dispose();
					_Watcher = null;
				}
				disposed = true;
			}
		}
		/// <summary>
		/// Dispose of the object
		/// </summary>
		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		#endregion
	}
}
