﻿/*
 * mDownloaderService - © EUROPEAN COMMISSION, 2009-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la
«Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que
conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse
suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le
logiciel distribué sous la Licence est distribué «en
l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT,
expresses ou implicites.
 * Consultez la Licence pour les autorisations et les
restrictions linguistiques spécifiques relevant de la
Licence.
 *
 * Auteur de l'oeuvre: Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 *
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Net;
using System.Net.Sockets;
using System.IO;

using System.Runtime.Serialization;

namespace mDownloaderService
{
	[Serializable()]
	public class mWebDocumentRequestSettings
	{
		public mWebDocumentRequestSettings()
		{
			init(null, true, 3, 100000);
		}
		public mWebDocumentRequestSettings(bool allowAutoRedirect, int maxRedirects)
		{
			init(null, allowAutoRedirect, maxRedirects, 100000);
		}
		public mWebDocumentRequestSettings(bool allowAutoRedirect, int maxRedirects, int timeout )
		{
			init(null, allowAutoRedirect, maxRedirects, timeout);
		}
		public mWebDocumentRequestSettings(IWebProxy proxy, bool allowAutoRedirect, int maxRedirects)
		{
			init(proxy, allowAutoRedirect, maxRedirects, 100000);
		}
		public mWebDocumentRequestSettings(IWebProxy proxy, bool allowAutoRedirect, int maxRedirects, int timeout)
		{
			init(proxy, allowAutoRedirect, maxRedirects, timeout);
		}
		protected void init(IWebProxy proxy, bool allowAutoRedirect, int maxRedirects, int timeout)
		{
			Proxy = (WebProxy)proxy;
			AllowAutoRedirect = allowAutoRedirect;
			MaximumAutomaticRedirections = maxRedirects;
			Timeout = timeout;
		}

		/// <summary>
		/// Gets or sets proxy information for the request.
		/// </summary>
		public WebProxy Proxy;
		/// <summary>
		/// Gets or sets a value that indicates whether the request should follow redirection responses.
		/// </summary>
		public bool AllowAutoRedirect = true;
		/// <summary>
		/// Gets or sets the maximum number of redirects that the request follows.
		/// </summary>
		public int MaximumAutomaticRedirections = 3;
		/// <summary>
		/// Gets or sets the time-out value in milliseconds for the GetResponse and GetRequestStream methods.
		/// </summary>
		public int Timeout = 100000;
	}

	[Serializable()]
	public class mWebDocument
	{
		/// <summary>
		/// Get the content of the exception in case of an error
		/// </summary>
		public Exception LastException { get { return _LastException;  } }
		private Exception _LastException;

		public bool Success { get { return null == _LastException; } }

		/// <summary>
		/// Get the method that is used to encode the body of the response
		/// </summary>
		public string ContentEncoding { get { return _ContentEncoding; } }
		private string _ContentEncoding;

		/// <summary>
		/// Get the Character Set of the response
		/// </summary>
		public Encoding CharacterSet{get {return _CharacterSet;} }
		private Encoding _CharacterSet;

		/// <summary>
		/// Get the name of the Character set of the response
		/// </summary>
		public string CharacterSetName {get{return _CharacterSetName; }}
		private string _CharacterSetName;

		/// <summary>
		/// Check if the response provides Character Set information
		/// </summary>
		public bool HasCharacterSet { get { return string.IsNullOrEmpty(_CharacterSetName); } }

		/// <summary>
		/// Check if the response provides a Character Set supported by the local system
		/// </summary>
		public bool HasSupportedCharacterSet { get { return null != _CharacterSet; }}
		
		/// <summary>
		/// Indicates if the response's content was automatically converted to unicode
		/// </summary>
		public bool WasCharacterSetConverted { get {return _WasCharacterSetConverted; }}
		private bool _WasCharacterSetConverted = false;

		/// <summary>
		/// Get the length of the content returned by the request
		/// </summary>
		public long ContentLength { get { return _ContentLength; } }
		private long _ContentLength;

		/// <summary>
		/// Get the length of the content returned by the request
		/// </summary>
		public string ContentType{get{return _ContentType;}}
		private string _ContentType;

		/// <summary>
		/// Check if the Content-Type represents a textual type of data (text/plain, text/xml, text/html, ...)
		/// </summary>
		public bool IsTextContentType
		{
			get
			{
				return string.IsNullOrEmpty(_ContentType) ? false : _ContentType.StartsWith("text/");
			}
		}

		/// <summary>
		/// Get the length of the content returned by the request
		/// </summary>
		public CookieCollection Cookies { get { return _Cookies; } }
		private CookieCollection _Cookies;

		/// <summary>
		/// Get the headers that are associated with this response from the server
		/// </summary>
		public WebHeaderCollection Headers { get { return _Headers; } }
		private WebHeaderCollection _Headers;

		/// <summary>
		/// Get the content returned by the request
		/// </summary>
		/// <remarks>
		/// Encoding is automatically converted to Unicode for text documents
		/// </remarks>
		public byte[] Body { get { return document; } }
		public byte[] BodyUtf8 { get { return Encoding.Convert(Encoding.Unicode, Encoding.UTF8, document); } }
		public override string  ToString() {
 			 return Encoding.Unicode.GetString(document);
		}

		// Body of the downloaded document
		private byte[] document;

		/// <summary>
		/// Gets the total duration of the request
		/// </summary>
		public TimeSpan Duration { get { return _Duration;  } }
		private TimeSpan _Duration;

		/*
		/// <summary>
		/// Body of the downloaded document as a string (Encoding is converted to unicode)
		/// </summary>
		public string UnicodeBody { 
			get { 
				if( null == _CharacterSet ) {
					return Encoding.Unicode.GetString( document );
				}
				return Encoding.UTF8.GetString(document);
				// return Encoding.Unicode.GetString( Encoding.Convert(_CharacterSet, Encoding.Unicode, document));
			} 
		}
		*/

		/// <summary>
		/// Gets a Boolean value that indicates whether this response was obtained from the cache.
		/// </summary>
		public bool IsFromCache { get { return _IsFromCache; } }
		private bool _IsFromCache;

		/// <summary>
		/// Gets a Boolean value that indicates whether both client and server were authenticated.
		/// </summary>
		public bool IsMutuallyAuthenticated { get { return _IsMutuallyAuthenticated; } }
		private bool _IsMutuallyAuthenticated;

		/// <summary>
		/// Gets the last date and time that the contents of the response were modified.
		/// </summary>
		public DateTime LastModified { get { return _LastModified; } }
		private DateTime _LastModified;

		/// <summary>
		/// Gets the method that is used to return the response.
		/// </summary>
		public string Method { get { return _Method; } }
		private string _Method;

		/// <summary>
		/// Gets the version of the HTTP protocol that is used in the response.
		/// </summary>
		public Version ProtocolVersion { get { return _ProtocolVersion; } }
		private Version _ProtocolVersion;

		/// <summary>
		/// Gets the URI of the Internet resource that responded to the request.
		/// </summary>
		public Uri ResponseUri { get { return _ResponseUri; } }
		private Uri _ResponseUri;

		/// <summary>
		/// Gets the name of the server that sent the response.
		/// </summary>
		public string Server { get { return _Server; } }
		private string _Server;

		/// <summary>
		/// Gets the status of the response.
		/// </summary>
		public HttpStatusCode StatusCode { get { return _StatusCode; } }
		private HttpStatusCode _StatusCode;

		/// <summary>
		/// Gets the status description returned with the response.
		/// </summary>
		public string StatusDescription { get { return _StatusDescription; } }
		private string _StatusDescription;

		// Private constructor... use httpGet() or httpPost() methods to factory an object
		private mWebDocument()
		{
		}

		/// <summary>
		/// Perform a HTTP GET request
		/// </summary>
		/// <param name="uri">URI of the document</param>
		/// <returns></returns>
		public static mWebDocument httpGet(string uri)
		{
			return httpRequest(method.Get, new Uri(uri), null, null);
		}

		/// <summary>
		/// Perform a HTTP POST request
		/// </summary>
		/// <param name="uri">URI of the document</param>
		/// <param name="postData"></param>
		/// <returns></returns>
		public static mWebDocument httpPost(string uri, byte[] postData)
		{
			return httpRequest(method.Post, new Uri(uri), postData, null);
		}

		/// <summary>
		/// Perform a HTTP GET request
		/// </summary>
		/// <param name="uri">URI of the document</param>
		/// <returns></returns>
		public static mWebDocument httpGet(Uri uri)
		{
			return httpRequest(method.Get, uri, null, null);
		}

		/// <summary>
		/// Perform a HTTP POST request
		/// </summary>
		/// <param name="uri">URI of the document</param>
		/// <param name="postData"></param>
		/// <returns></returns>
		public static mWebDocument httpPost(Uri uri, byte[] postData)
		{
			return httpRequest(method.Post, uri, postData, null);
		}

		/// Perform a HTTP GET request
		/// </summary>
		/// <param name="uri">URI of the document</param>
		/// <param name="settings">Request settings</param>
		/// <returns></returns>
		public static mWebDocument httpGet(string uri, mWebDocumentRequestSettings settings)
		{
			return httpRequest(method.Get, new Uri(uri), null, settings);
		}

		/// <summary>
		/// Perform a HTTP POST request
		/// </summary>
		/// <param name="uri">URI of the document</param>
		/// <param name="postData"></param>
		/// <param name="settings">Request settings</param>
		/// <returns></returns>
		public static mWebDocument httpPost(string uri, byte[] postData, mWebDocumentRequestSettings settings)
		{
			return httpRequest(method.Post, new Uri(uri), postData, settings);
		}

		/// <summary>
		/// Perform a HTTP GET request
		/// </summary>
		/// <param name="uri">URI of the document</param>
		/// <param name="settings">Request settings</param>
		/// <returns></returns>
		public static mWebDocument httpGet(Uri uri, mWebDocumentRequestSettings settings)
		{
			return httpRequest(method.Get, uri, null, settings);
		}

		/// <summary>
		/// Perform a HTTP POST request
		/// </summary>
		/// <param name="uri">URI of the document</param>
		/// <param name="postData"></param>
		/// <param name="settings">Request settings</param>
		/// <returns></returns>
		public static mWebDocument httpPost(Uri uri, byte[] postData, mWebDocumentRequestSettings settings)
		{
			return httpRequest(method.Post, uri, postData, settings);
		}

		// Http Methods
		protected enum method{ Get, Post };

		// Perform an HTTP request
		protected static mWebDocument httpRequest(method httpMethod, Uri uri, byte[] requestData, mWebDocumentRequestSettings settings)
		{
			mWebDocument doc = new mWebDocument();
			HttpWebRequest request;
			HttpWebResponse response = null;
			DateTime tmStart = DateTime.Now;
			DateTime tmEndDownload;
			DateTime tmEndAll;

			doc._ResponseUri = uri;

			// Build the Request Object
			try {
				request = (HttpWebRequest)HttpWebRequest.Create(uri);
				if (null != settings)
				{
					if (null != settings.Proxy)
					{
						request.Proxy = settings.Proxy;
					}
					request.MaximumAutomaticRedirections = settings.MaximumAutomaticRedirections;
					request.AllowAutoRedirect = settings.AllowAutoRedirect;
					request.Timeout = settings.Timeout;
				}
				if (method.Post == httpMethod)
				{
					request.Method = "POST";
					request.ContentType = "application/x-www-form-urlencoded";
				}
				else
				{
					request.Method = "GET";
				}
				if (null != requestData)
				{
					request.ContentLength = requestData.Length;
					Stream strm = request.GetRequestStream();
					strm.Write(requestData, 0, requestData.Length);
					strm.Close();
				}
				
				request.CookieContainer = new CookieContainer();
			} catch( Exception e ) {
				doc._LastException = e;
				return doc;
			}

			// Perform the Request
			try
			{
				response = (HttpWebResponse)request.GetResponse();
				tmEndDownload = DateTime.Now;

				// Fetch the Response (Copy the content then dispose the Response object)
				doc._ContentEncoding = response.ContentEncoding;
				doc._ContentLength = response.ContentLength;
				doc._ContentType = response.ContentType;
				doc._Headers = response.Headers;
				doc._Cookies = response.Cookies;
				doc._Duration = tmEndDownload - tmStart;
				doc._CharacterSetName = response.CharacterSet;
				doc._Cookies = response.Cookies;
				doc._Headers = response.Headers;
				doc._IsFromCache = response.IsFromCache;
				doc._IsMutuallyAuthenticated = response.IsMutuallyAuthenticated;
				doc._LastModified = response.LastModified;
				doc._Method = response.Method;
				doc._ProtocolVersion = response.ProtocolVersion;
				doc._ResponseUri = response.ResponseUri;
				doc._Server = response.Server;
				doc._StatusCode = response.StatusCode;
				doc._StatusDescription = response.StatusDescription;
				if (!string.IsNullOrEmpty(doc._CharacterSetName))
				{
					try
					{
						doc._CharacterSet = Encoding.GetEncoding(doc._CharacterSetName);
					}
					catch (ArgumentException)
					{
						// unsupported character set
					}
				}

				// Read the data
				if (doc.IsTextContentType && doc.HasSupportedCharacterSet)
				{
					// Text data, convert to unicode encoding
					Stream respStream = response.GetResponseStream();
					StreamReader respReader = new StreamReader(respStream, doc.CharacterSet);
					string stringDocument = respReader.ReadToEnd();
					respReader.Close();
					if (Encoding.Unicode.Equals(doc.CharacterSet))
					{
						doc._WasCharacterSetConverted = false;
						doc.document = Encoding.Unicode.GetBytes(stringDocument);
					}
					else
					{
						doc._WasCharacterSetConverted = true;
						doc.document = Encoding.Convert(doc.CharacterSet, Encoding.Unicode, doc.CharacterSet.GetBytes(stringDocument));
					}
				}
				else
				{
					// Binary data
					doc._WasCharacterSetConverted = false;
					Stream respStream = response.GetResponseStream();
					BinaryReader respReader = new BinaryReader(respStream);
					System.IO.MemoryStream ms = new System.IO.MemoryStream();
					int bufferSize = 4096;
					byte[] buffer;
					do
					{
						buffer = respReader.ReadBytes(bufferSize);
						ms.Write(buffer, 0, buffer.Length);
					} while (buffer.Length == bufferSize);
					respReader.Close();
					doc.document = ms.ToArray();
				}

				tmEndAll = DateTime.Now;
				doc._Duration = tmEndAll - tmStart;
			}
			catch (WebException e)
			{
				doc._LastException = e;
				if (WebExceptionStatus.ProtocolError == e.Status)
				{
					doc._StatusCode = ((HttpWebResponse)e.Response).StatusCode;
					doc._StatusDescription = ((HttpWebResponse)e.Response).StatusDescription;
				}
			}
			catch (Exception e)
			{
				doc._LastException = e;
			}
			finally
			{
				if (null != response)
				{
					response.Close();
					response = null;
				}
			}

			return doc;
		}

/*		public mWebDocument httpGetAsynch(string url)
		{
			return httpRequestAsynch(method.Get, url, null);
		}
		public mWebDocument httpPostAsynch(string url, byte[] postData)
		{
			return httpRequestAsynch(method.Post, url, postData);
		}
		protected mWebDocument httpRequestAsynch(method httpMethod, string url, byte[] requestData)
		{
			mWebDocument doc = new mWebDocument();
			return doc;
		}
*/
	}
}
