﻿/*
 * © EUROPEAN COMMISSION, 2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous
 * la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou 
 * implicites.
 * Consultez la Licence pour les autorisations et les restrictions 
 * linguistiques spécifiques relevant de la Licence.
 *
 * Auteur de l'oeuvre: Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.ServiceProcess;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Configuration;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.IO.Pipes;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters;
using System.Runtime.Serialization.Formatters.Binary;

using MetaNull.Console;
using MetaNull.Diagnostics;
using MetaNull.IO;
using MetaNull.Threading;

namespace mFileMonitorService
{
	/// <summary>
	/// mFileMonitor Windows Service
	/// </summary>
	public partial class mFileMonitorService : ServiceBase
	{

		/// <summary>
		/// Service's Main Thread
		/// </summary>
		Thread thMainThread;

		/// <summary>
		/// Status reporting thread
		/// </summary>
		Thread thStatusThread;

		/// <summary>
		/// Event triggered by the service's OnStop() method when the process should terminate itself
		/// </summary>
		private static ManualResetEvent stopService;

		/// <summary>
		/// Counters (for the status thread)
		/// </summary>
		CounterSet Counters = new CounterSet();

		/// <summary>
		/// FileSystem Monitors (one for each input folder)
		/// </summary>
		List<FilesystemMonitor> monitors;

		/// <summary>
		/// Constructor
		/// </summary>
		public mFileMonitorService()
		{
			InitializeComponent();
			CanStop = true;
			CanShutdown = true;
			CanPauseAndContinue = false;

			monitors = new List<FilesystemMonitor>();
		}

		/// <summary>
		/// Called on Service startup
		/// </summary>
		/// <param name="args"></param>
		protected override void OnStart(string[] args)
		{
#if DEBUG
			Debugger.Launch();
#endif
			LoadConfiguration();

			stopService = new ManualResetEvent(false);

			foreach(Input input in inputFolders )
			{
				try
				{
					Counters.Increment("MonitoredFolders");
					FileMover fm = new FileMover(input, outputFolders);
					fm.OnFileCreated += new FilesystemMonitor.FileCreatedHandler(fm_OnFileCreated);
					fm.OnFileDeleted += new FilesystemMonitor.FileDeletedHandler(fm_OnFileDeleted);
					fm.OnFileRenamed += new FilesystemMonitor.FileRenamedHandler(fm_OnFileRenamed);
					fm.OnThreadError += new FilesystemMonitor.ThreadErrorHandler(fm_OnThreadError);
					fm.OnFilesystemError += new FileMover.FilesystemErrorHandler(fm_OnFilesystemError);
					fm.OnInputFileCopied += new FileMover.InputFileCopiedHandler(fm_OnInputFileCopied);
					fm.OnInputFileDeleted += new FileMover.InputFileDeletedHandler(fm_OnInputFileDeleted);
					fm.OnInputFileProcessing += new FileMover.InputFileProcessingHandler(fm_OnInputFileProcessing);
					fm.OnInputFileProcessed += new FileMover.InputFileProcessedHandler(fm_OnInputFileProcessed);
					fm.OnProcessingError += new FileMover.ProcessingErrorHandler(fm_OnProcessingError);
					monitors.Add(fm);
				}
				catch (ArgumentException e)
				{
					eventLog.WriteEntry(e.ToString(), EventLogEntryType.Error);
				}
			}

			ParameterizedThreadStart startStThread = new ParameterizedThreadStart(ThreadStatus);
			thStatusThread = new Thread(startStThread);
			thStatusThread.Start(new ThreadParameter());

			ThreadStart fct = new ThreadStart(ThreadMain);
			thMainThread = new Thread(fct);
			thMainThread.Start();
		}

		/// <summary>
		/// Called on when service is requested to stop
		/// </summary>
		protected override void OnStop()
		{
			for (int m = 0; m < monitors.Count; m++)
			{
				monitors[m].MonitoringEnabled = false;
				monitors[m].Dispose();
			}
			monitors.Clear();

			stopService.Set();

			if (thStatusThread.IsAlive)
			{
				thStatusThread.Join(1000);
			}
			if (thMainThread.IsAlive)
			{
				thMainThread.Join(5000);
			}
 		}

		/// <summary>
		/// Main thread of the service
		/// </summary>
		private void ThreadMain()
		{
			bool commitSuicide = false;
			try
			{
				// Enable monitoring
				for (int m = 0; m < monitors.Count; m++)
				{
					monitors[m].MonitoringEnabled = true;
				}

				// Wait for the service to receive a quit request
				while (false == stopService.WaitOne(cfgYieldTime))
				{
					// Nothing to do (Actions performed by the monitors themselves)
				}
			}
			catch (Exception e)
			{
				eventLog.WriteEntry("MainThread: An exception has occured " + e.ToString(), EventLogEntryType.Error);
				commitSuicide = true;
			}
			finally
			{
				stopService.Set();
			}

			if (commitSuicide)
			{
				Environment.Exit(1);
			}
			// Thread.CurrentThread.Abort();
		}

		/// <summary>
		/// Thread responsible of sending status notification to the listeners (if any)
		/// </summary>
		/// <param name="param"></param>
		private void ThreadStatus(object param)
		{
			ThreadParameter arguments = (ThreadParameter)param;
			String threadSignature = "StatusThread#" + arguments.InstanceNumber;
			Counters.Increment("ThreadStatus");
			if (true == cfgReportStatus)
			{
				DateTime lastRefresh = new DateTime(0);
				while (!stopService.WaitOne(cfgYieldTime))
				{
					try
					{
						if (0 == lastRefresh.Ticks || 1000 < (DateTime.Now - lastRefresh).TotalMilliseconds)
						{
							// Serialize the "Counters" object and send it to the listeners
							using (NamedPipeClientStream pipe = new NamedPipeClientStream(cfgStatusPipeServer, cfgStatusPipeName, PipeDirection.Out))
							{
								pipe.Connect(0);
								if (pipe.CanWrite)
								{
									BinaryFormatter bf = new BinaryFormatter();
									bf.Serialize(pipe, Counters.GetCountersCopy());
									pipe.Flush();
								}
							}
							lastRefresh = DateTime.Now;
						}
					}
					catch (System.TimeoutException)
					{
						// Nothing to do (pipe connection lost)
					}
					catch (System.Exception e)
					{
						eventLog.WriteEntry(threadSignature + ": " + e.ToString(), EventLogEntryType.Error);
					}
				}
			}
			Counters.Decrement("ThreadStatus");

			// Sends a final status report to the status thread
			try
			{
				using (NamedPipeClientStream pipe = new NamedPipeClientStream(cfgStatusPipeServer, cfgStatusPipeName, PipeDirection.Out))
				{
					pipe.Connect(0);
					if (pipe.CanWrite)
					{
						BinaryFormatter bf = new BinaryFormatter();
						bf.Serialize(pipe, Counters.GetCountersCopy());
						pipe.Flush();
					}
				}
			}
			catch (System.Exception)
			{
				//Swallow the exception
			}
		}

		void fm_OnFileRenamed(string OldFullPath, string NewFullPath)
		{
			Counters.Increment("RenamedFiles");
#if DEBUG
			eventLog.WriteEntry("File Renamed: " + Path.GetFileName(OldFullPath) + " > " + Path.GetFileName(NewFullPath), EventLogEntryType.Information);
#endif
		}
		void fm_OnFileCreated(string FullPath)
		{
			Counters.Increment("CreatedFiles");
#if DEBUG
			eventLog.WriteEntry("New file: " + Path.GetFileName(FullPath), EventLogEntryType.Information);
#endif

		}
		void fm_OnFileDeleted(string FullPath)
		{
			Counters.Increment("DeletedFiles");
#if DEBUG
			eventLog.WriteEntry("File deleted: " + Path.GetFileName(FullPath), EventLogEntryType.Information);
#endif
		}
		void fm_OnFilesystemError(Exception Error)
		{
			Counters.Increment("Errors");
			eventLog.WriteEntry("Filesystem error: " + Error.ToString(), EventLogEntryType.Warning);
		}

		void fm_OnProcessingError(Exception Error)
		{
			Counters.Increment("Errors");
			eventLog.WriteEntry("Processing error: " + Error.ToString(), EventLogEntryType.Warning);
		}

		void fm_OnInputFileProcessed(string path, string processPath, string processArguments, int exitCode, string processOutput, string processError)
		{
			Counters.Decrement("InputFile Processing");
			Counters.Increment("InputFile Processed");

			eventLog.WriteEntry("File processed\r\nOutput:\r\n " + processOutput + "\r\nError:\r\n" + processError);
		}

		void fm_OnInputFileProcessing(string path, string processPath, string processArguments)
		{
			Counters.Increment("InputFile Processing");
			eventLog.WriteEntry("Processing file : " + path + " with command [" + processPath + " " + processArguments + "]", EventLogEntryType.Information);
		}

		void fm_OnInputFileDeleted(string path)
		{
			Counters.Increment("InputFile Deleted");
		}

		void fm_OnInputFileCopied(string path)
		{
			Counters.Increment("InputFile Copied");
		}

		/// <summary>
		/// Fatal error in an inner thread...
		/// </summary>
		/// <param name="Error"></param>
		void fm_OnThreadError(Exception Error)
		{
			Counters.Increment("FatalErrors");
			eventLog.WriteEntry("Internal thread error: " + Error.ToString(), EventLogEntryType.Error);
			stopService.Set();
		}
	}
 
}
