<?php
/*
 * © EUROPEAN COMMISSION, 2007-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou implicites.
 * Consultez la Licence pour les autorisations et les restrictions linguistiques spécifiques relevant de la Licence.
 *
 * Auteurs:
 *	Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 *	Cosmin Popescu,   Adrian.POPESCU@ext.ec.europa.eu
 */


namespace MetaNull\Application {

	/**
	 * Base class defining the mechanism of a modular application
	 * @author Pascal Havelange
	 * @copyright © EUROPEAN COMMISSION, 2008
	 * @package MetaNull\Application
	 * @license http://ec.europa.eu/idabc/eupl
	 */
	abstract class mModularApplication extends mApplication {

		/**
		 * The list of known modules
		 * @var array An array of mModule objects
		 */
		protected $modules = array();

		/**
		 * The index of the default module, in the array of modules
		 */
		protected $default_module = false;

		/**
		 * Object Constructor
			 * @param array $application_config The application params. The array should be an associative array
			 *				If you want to send the config path and the config format just send an array like array('path' => 'my_path', 'format' => 'my_format')
			 *				If you want to send the config array directly, just send an array like array('config' => $my_config)
		 * @param bool $auto_save_config If set to true, the application will automatically save the configuration file on exit
		 */
		public function __construct( $application_config ) {
			parent::__construct( $application_config );
			$this->modules = array();
			$this->default_module = false;
		}

		/**
		 * It is called prior to the execution of the module
		 * Throw an Exception from this method to immediatelly stop the application (in which case onDestroy() will be gracefuly called)
		 * @access protected
		 */
		protected function onInitialize() {
			parent::onInitialize();
		}

		/**
		 * This method is called last (after the execution of the module)
		 * No exception should be thrown within this method, its only goal is to possibly free some resources
		 * @access protected
		 */
		protected function onDestroy() {
			parent::onDestroy();
		}

		/**
		 * This method must be implemented in the derivated class
		 * This method is called whenever an exceptions occurs in onInitialize or onExecute
		 * Please note that the onDestroy() method will still be called afterwards.
		 * @access protected
		 */
		protected function onException( $e ) {
			parent::onException( $e );
		}

		/**
		 * Main logic of the application: Look for the appropriate module in the list, and delegates the execution to the module.
		 * This method will call the onExecuteModule() method
		 * @access protected
		 * @final
		 */
		final protected function onExecute() {
			$module = null;
			for( $module_idx = 0; $module_idx < count($this->modules); $module_idx ++ ) {
				if( $module_idx === $this->default_module ) {
					// The default module is able to process any request... so skip it at this time
					continue;
				}
				/** @var mModule $module */
				$module = &$this->modules[$module_idx];
				if( $module->canProcessRequest() ) {
					break;
				}
			}
			if ($module_idx == count($this->modules)){
				if( 0 == count( $this->modules ) || false === $this->default_module || !isset($this->modules[$this->default_module]) ) {
					throw new noDefaultModuleException();
				}
				$module = &$this->modules[$this->default_module];
			}

			$module_result = $module->run( $this->config );
			if( $module->hasUpdatedConfiguration() ) {
				$this->config = $module->getConfiguration();
			}
			$module_info = $module->getInfo();

			$module_template = (null === $module->getDefaultTemplateFile() ? self::default_template_file : $module->getDefaultTemplateFile());
			$module_headers = $module->getHeaders();
			$this->onExecuteModule( $module_result, $module_info, $module_template, $module_headers );
		}

		/**
		 * Add a mModule object to the list of modules of the application
		 * @param mModule $module The module object
		 * @param bool $is_default If set to true, set the new module as the default module
		 */
		public function addModule( $module, $is_default = false ) {
			$count = count($this->modules);

			$this->modules[] = $module;
			if( false !== $is_default ) {
				$this->default_module = $count;
			}
		}

		/**
		 * Get the list of modules
		 * @return mModule[]
		 */
		public function getModules() {
			return $this->modules;
		}

		/**
		 * Main Template of the application
		 * @var MTemplate
		 */
		protected $template = false;

		/**
		 * Base name of the folder used by this application
		 * (Used to locate the files, and to locate the templates)
		 * @var string
		 */
		protected $base_folder = false;

		/**
		 * Default name of the maint template file
		 * @var string
		 */
		const default_template_file = 'index.html';

		/**
		 * This method is called after the execution of a module, allowing the application to draw itself, including module's results.
		 $ Throw an exception in this method to terminate the application
		 * @access protected
		 * @param string $module_results The resulting output of the module's execution
		 * @param array $module_info An indexed array of strings containing informations about the module
		 * @param string $module_template Default template file (default = index.html)
		 * @param array|null $module_headers Null or headers set by the module
		 */
		protected function onExecuteModule( $module_result, $module_info, $module_template = self::default_template_file, $module_headers = null ) {
			$this->template = new MTemplate( 'm' );
			$this->template->loadTemplate( $this->getTemplateFilePath( $module_template ) );
			$this->template->data = $module_info;
			$this->template->data['module-result'] = $module_result;

			$content_type = false;
			foreach( headers_list() as $header ) {
				list( $key, $val ) = explode( ':', $header, 2 );
				$val;	// not used;
				if( strtolower( $key ) == 'content-type' ) {
					$content_type = true;
					break;
				}
			}
			if( false === $content_type ) {
				if( isset( $this->template->data['content-type'] )) {
					header( "Content-Type: {$this->template->data["content-type"]}; charset={$this->template->data['charset']}" );
				} else {
					header( "Content-Type: text/html; charset={$this->template->data['charset']}" );
				}
			}
			if( is_array( $module_headers )) {
				foreach( $module_headers as $header ) {
					header( $header );
				}
			}
			echo $this->template->transform();
		}

		/**
		 * Returns the Full path to the templates folder of this application (no trailing slash)
		 * @access public
		 * @return string
		 */
		public final function getTemplatesFolder() {
			return "{$this->config['templates']['base']}/{$this->config['templates']['default']}/{$this->base_folder}";
		}
		/**
		 * Returns the full path to a specific template file of the application
		 * @param string $template_file Template file's name
		 * @return string
		 * @access public
		 */
		public final function getTemplateFilePath( $template_file ) {
			$path = "{$this->config['templates']['base']}/{$this->config['templates']['default']}/";
			if( !empty($this->base_folder)){
				$path .= "{$this->base_folder}/";
			}
			$path .= $template_file;
			return $path;
		}
	}

}
?>