<?php
/*
 * © EUROPEAN COMMISSION, 2007-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou implicites.
 * Consultez la Licence pour les autorisations et les restrictions linguistiques spécifiques relevant de la Licence.
 *
 * Auteurs:
 *	Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 *	Cosmin Popescu,   Adrian.POPESCU@ext.ec.europa.eu
 */

namespace MetaNull\Application {

	/**
	 * Base class defining modulare mechnaism for applications
	 * Each module is responsible for the display of its data
	 * It has a canProcessRequest() method the application can use to disocver which module to load
	 * @author Pascal Havelange
	 * @copyright © EUROPEAN COMMISSION, 2008
	 * @package MetaNull\Application
	 * @license http://ec.europa.eu/idabc/eupl
	 */
	abstract class mModule {

		/**
		 * Module's title
		 * @var string
		 */
		protected $title = '';

		/**
		 * Module's description
		 * @var string
		 */
		protected $description = '';

		/**
		 * Reference to the application's configuration (set through onInitialize())
		 * @var array
		 */
		protected $config = array();

		/**
		 * Constructor
		 * @param array $config A multidimentionnal array containing the configuration
		 */
		public function __construct( $config ) {
			$this->title = '';
			$this->description = '';
			$this->config = $config;
		}

		/**
		 * Execute the module and returns its output
		 * @return string
		 */
		final public function run() {
			$exception = null;
			$output = null;

			try {
				$this->onInitialize();
				$output = $this->onExecute();
			}
			catch(\MetaNull\Tools\mysqlException $e){
				$exception = $e;
			} catch( \Exception $e ) {
				$exception = $e;
			}
			$this->onDestroy();
			if( is_a( $exception, 'Exception' )) {
				throw $exception;
			}
			return $output;
		}

		/**
		 * Initializes the module. It is called prior to onExecute()
		 * You might override this method.
		 * Throw an exception in this method to abort module's execution
		 */
		protected function onInitialize() {
		}

		/**
		 * Free resources used by the module. It is called after onExecute() (or after an exception has been caught)
		 * You might override this method.
		 * This method should not throw any exception
		 */
		protected function onDestroy() {

		}

		/**
		 * Core logic of the module
		 * Place here the code of your module, and return the data that is to be displayed in the application
		 * @return string
		 */
		abstract protected function onExecute();

		/**
		 * Request the module to test if it can handle the current request ($_POST, $_GET, $_SERVER['argv'])
		 * @return bool True if the module can handle the request, false otherwise
		 */
		abstract public function canProcessRequest();

		/**
		 * If the modules modifies the configuration, this method should return true to allow the caller application to modify the configuration file
		 * @return bool
		 */
		abstract public function hasUpdatedConfiguration();

		/**
		 * Returns the configuration array
		 * If hasUpdatedConfiguration() returns true, the caller application will use this method to get a copy of the new configuration.
		 * Therefore, the onDestroy() method should NEVER erase the $config member variable (This is not an issue anyway, as it is only a member variable, that get's automatically destroyed with the object)
		 * @return array
		 */
		final public function getConfiguration() {
			return $this->config;
		}

		/**
		 * Returns an associative array of strings giving information about the module
		 * @return array
		 */
		public function getInfo() {
			return array( 'title' => $this->title, 'description' => $this->description );
		}

		/**
		 * Returns the user defined headers of the application
		 * @access public
		 * @return array|null
		 */
		public function getHeaders() {
			return null;
		}

		/**
		 * Returns the application default template file
		 * @access public
		 * @return string|null
		 */
		public function getDefaultTemplateFile() {
			return null;
		}
	}

}
?>