<?php
/*
 * © EUROPEAN COMMISSION, 2007-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou implicites.
 * Consultez la Licence pour les autorisations et les restrictions linguistiques spécifiques relevant de la Licence.
 *
 * Auteurs:
 *	Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 *	Cosmin Popescu,   Adrian.POPESCU@ext.ec.europa.eu
*/

namespace MetaNull\Commission {

/**
 * EC_User stores the information about an user of the application
 * iUser implementation, focussed on the usage of the ECAS system
 * The commission's LDAP database is used to retrieve user attributes
 * @author Pascal Havelange
 * @copyright © EUROPEAN COMMISSION, 2008
 * @package MetaNull\Commission
 * @license http://ec.europa.eu/idabc/eupl
 */
class EC_User implements \MetaNull\Authentication\iUser {

    /**
     * ECAS User ID
     * @var string
     */
    public $user_id = false;

    /**
     * LDAP Attributes
     * @var LDAPEntry
     */
    public $ldap_attributes = null;

    /**
     * The user preferences
     * @var array
     * @access public
     */
    protected $user_preferences = array();

    /**
     * The RELEX NG access
     * @var string

     */
    const ACCESS_NG = 'NG';

    /**
     * The RELEX MS access
     * @var string

     */
    const ACCESS_MS = 'MS';

    /**
     * The RELEX EC access
     * @var string

     */
    const ACCESS_EC = 'EC';

    /**
     * The RELEX RX access
     * @var string

     */
    const ACCESS_RX = 'RX';

    /**
     * The list of engines to which the user has access
     * @var string[]
     * @access private
     */
    private $engines;

    /**
     * The access level of the user
     * @var string
     * @access private
     */
    private $access_level = 'NG';

    /**
     * Constructor
     * @param string $user_id UserID
     * @param bool $fetch_ldap If set to true, tries to retrieve LDAP Attributes for the user
     * @param array $groups A list of groups (if the attributes are not fetched through LDAP, the list of groups can be specified here)
     */
    public function __construct( $user_id, $fetch_ldap = true, $groups = null ) {
	$this->user_id = $user_id;
	$this->ldap_attributes = array();
	if( $fetch_ldap ) {
	    $ldap_query = "(&(objectClass=person)(uid={$user_id}))";
	    $this->ldap_attributes = EC_User::fetchLDAP( $user_id );
	}
	else {
	    $this->ldap_attributes = array(
		    'uid' => $user_id,
		    'mail' => '',
		    'cn' => $user_id,
	    );

	    if ($groups != null) {
		$this->ldap_attributes['cudgroup'] = array();
		foreach ($groups as $group) {
		    $this->ldap_attributes['cudgroup'][] = $group;
		}
	    }
	}
	
	if ($this->isMemberOf('RELEX_TARIQARX', false)) {
	    $this->access_level = 'RX';
	}
	else if ($this->isMemberOf('EEAS_INTRANET', false)) {
	    $this->access_level = 'RX';
	}
	else if ($this->isMemberOf('RELEX_TARIQAEC', false)) {
	    $this->access_level = 'EC';
	}
	else if ($this->isMemberOf('RELEX_TARIQAEM', false)) {
	    $this->access_level = 'MS';
	}
    }

    /**
     * Returns the UserId of the user
     * @return string
     */
    public function getUserId() {
	return $this->user_id;
    }

    /**
     * Returns an array of valid attributes associated with
     * @return array
     */
    public function getAttributeList() {
	return array_keys( EC_LDAP::$ldap_attributes );
    }

    /**
     * Check the group membership of an user
     * @param string|array $groups Either the name of a group or an array of group names against which to check the membership of the user
     * @param bool $binary_and If set to true, the user must be a member of ALL the groups in the list, otherwise it must belong to at least one of the groups
     * @return bool
     */
    public function isMemberOf( $groups, $binary_and = false ) {
	if( 0 == count( $this->ldap_attributes ) || false === $this->user_id  ) {
	    throw new \MetaNull\Authentication\authenticationException( 'Not authenticated/No LDAP attributes available' );
	}
	if( !is_array( $groups )) {
	    return \MetaNull\Net\mLDAP::testAttributeI( $this->ldap_attributes, 'cudgroup', $groups );
	} else {
	    if( false === $binary_and ) {
		foreach( $groups as $group ) {
		    if( \MetaNull\Net\mLDAP::testAttributeI( $this->ldap_attributes, 'cudgroup', $group )) {
			return true;
		    }
		}
		return false;
	    } else {
		foreach( $groups as $group ) {
		    if( !\MetaNull\Net\mLDAP::testAttributeI( $this->ldap_attributes, 'cudgroup', $group )) {
			return false;
		    }
		}
		return true;
	    }
	}
    }

    /**
     * Accessor, quickly access the user's preferences
     * @return string
     */
    public function __get( $attributeName ) {
	if( $this->__isset( $attributeName ) ) {
	    return $this->ldap_attributes[EC_LDAP::$ldap_attributes[$attributeName]];
	} else {
	    return false;
	    //throw new \MetaNull\Authentication\authenticationException( "LDAP Attribute '{$attributeName}' not found!" );
	}
    }

    /**
     * Accessor, quickly access the user's preferences
     */
    public function __set( $attributeName, $value ) {
	if( $this->__isset( $attributeName ) ) {
	    $this->ldap_attributes[EC_LDAP::$ldap_attributes[$attributeName]] = $value;
	} else {
	    //throw new \MetaNull\Authentication\authenticationException( "LDAP Attribute '{$attributeName}' not found!" );
	}
    }

    /**
     * Accessor, quickly access the user's attributes using object style syntax
     * @return bool
     */
    public function __isset( $attributeName ) {
	if( 0 == count( $this->ldap_attributes ) ||
		!isset(EC_LDAP::$ldap_attributes[$attributeName]) ||
		!isset( $this->ldap_attributes[EC_LDAP::$ldap_attributes[$attributeName]] )
	) {
	    return false;
	}
	return true;
    }

    /**
     * Accessor, quickly access the user's attributes using object style syntax
     * @return bool
     */
    public function __unset( $attributeName ) {
	if( $this->__isset( $attributeName ) ) {
	    unset( $this->ldap_attributes[EC_LDAP::$ldap_attributes[$attributeName]] );
	} else {
	    throw new \MetaNull\Authentication\authenticationException( "LDAP Attribute '{$attributeName}' not found!" );
	}
    }

    /**
     * Get user attributes from the LDAP database
     *
     * @param string $user_id Id of the user in the LDAP database
     * @return array An array of attributes
     * @static
     * @access public
     */
    static public function fetchLDAP( $user_id ) {
	$ldap = new EC_LDAP();
	if( false === $ldap->connect() ) {
	    throw new \MetaNull\Authentication\authenticationException( "Can't connect to the LDAP Server!" );
	}
	$results = $ldap->search( "(&(objectClass=person)(uid={$user_id}))" );
	if( 0 == count( $results ) || !isset( $results[0] )) {
	    throw new \MetaNull\Authentication\authenticationException( "User '{$user_id}' not found in the LDAP database!" );
	}
	return $results[0];
    }

    /**
     * Gets an user preference. Returns null if the preference is not set
     * @param string $name The name of the preference
     * @access public
     * @return mixed
     */
    public function get_preference($name) {
	if (array_key_exists($name, $this->user_preferences)) {
	    $value = $this->user_preferences[$name];
	    if ($value == 'true') {
		$value = true;
	    }
	    else if ($value == 'false') {
		$value = false;
	    }
	    return $value;
	}

	return null;
    }

    /**
     * Gets the user access level
     * @access public
     * @return int
     */
    public function get_access_level() {
	return $this->access_level;
    }

    /**
     * Sets an user preference.
     * To delete a preference from the database, just set the null value for it and set the $db to an instance of mMysql.
     * If you want to delete an array from the database, just put a "%" instead of the index value. For example,
     * to delete the "general/countries" array: $user->set_preference("general/countries/%", null);
     * @param string $name The preference
     * @param mixed $value The value
     * @param mMysql $db A database connection (used for saving the preference to the database) [OPTIONAL]
     * @access public
     * @return void
     */
    public function set_preference($name, $value, $db = null) {
	if ($value === true) {
	    $value = 'true';
	}
	else if ($value === 'false'){
	    $value = false;
	}
	if ($db != null) {
	    $_name = $db->real_escape_string($name);
	    if ($value == null) {
		if (substr($name, -1) == "%") {
		    $op = "like";
		}
		else {
		    $op = "=";
		}
		$sql = <<<sql
delete from `user_preferences`
where `key` {$op} '{$_name}' and `user_id` = '{$this->user_id}'
sql;

		$db->mQuery($sql);

		unset($this->user_preferences[$_name]);
	    }
	    else {
		$_value = $db->real_escape_string($value);
		$sql = "call set_preference('{$this->user_id}', '{$_name}', '{$_value}')";
		$db->mQuery($sql);
	    }
	}

	$pattern = '/^(.*)\/(\d*)$/i';

	$idx = preg_replace($pattern, '\2', $name);

	if (is_numeric($idx)) {
	    $this->user_preferences[preg_replace($pattern, '\1', $name)][$idx] = $value;
	}
	else {
	    $this->user_preferences[$name] = $value;
	}
    }
}

}
?>