<?php
/*
 * © EUROPEAN COMMISSION, 2007-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou implicites.
 * Consultez la Licence pour les autorisations et les restrictions linguistiques spécifiques relevant de la Licence.
 *
 * Auteurs:
 *	Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 *	Cosmin Popescu,   Adrian.POPESCU@ext.ec.europa.eu
 */

namespace tariqa3{

    /**
     * Class for handling the dojo store data
     * @package tariqa3
     * @author Cosmin Popescu
     * @copyright © EUROPEAN COMMISSION, 2009-2010
     * @license http://ec.europa.eu/idabc/eupl
     */
    class dojo_connector {
        /**
         * The store database connection
         * @var resource
         * @access protected
         */        
        protected $db = null;

        /**
         * The source file of the database json file
         * @var string
         * @access protected
         */
        
        protected $db_file;
        
        /**
         * Constructs the object
         * @param string $page_id The page identifier
         * @param string $db_file The source file of the database json file
         * @param array $config The configuration array
         * @access public
         * @return void
         */
        public function __construct($db, $db_file) {
            $this->db = $db;
            $this->db_file = $db_file;
        }

        /**
         * Gets the user input part from a json used to build the SQL
         * The json array should be the same json array used in the json2sql function.
         * The "user_input" section should be another json array in the form:
         * {'fields: {
         *            'variable1': [
         *                         {'as_part' => 'field_name'}, ... // If the variable is present in GET or POST, this part will be added to final sql
         *                         ],
         *            'variable2': ...
         *  },
         *  'tables': {
         *             'variable1': 'string representing the from part of the select',
         *             'variable2': ...
         *  },
         *  'where': {
         *            'variable1': 'string representing the where part of the select',
         *            'variable2': ... //Please notice the the function will not add the "and" or "or" parts. These should be present in the strings
         *  }, 
         *  'having': {
         *             'variable1': 'string representing the having part of the select',
         *             'variable2': ...//Please notice the the function will not add the "and" or "or" parts. These should be present in the strings
         *  },
         *  'order': {
         *            'variable1': [
         *                          'field', ...,
         *                         ],
         *            'variable2': ...
         *  }
         *  Any part is OPTIONAL, as the entire user_input section
         *  NOTE: If you want the use inputs to be escaped, then name the key with '__' as prefix. 
         * @param array $json The json array
         * @param string $search The part being searched (fields | tables | where | having | grouping | order | limit)
         * @static
         * @access public
         * @return array
         */
        public static function getJsonUserInputPart($json, $search){
            $result = array();
            if (array_key_exists('user_input', $json)){
                if (array_key_exists($search, $json['user_input'])){
                    foreach ($json['user_input'][$search] as $key => $value){
                        if (array_key_exists($key, $_GET) || array_key_exists($key, $_POST)){
                            $result[] = $value;
                        }
                    }
                }
            }

            return $result;
        }

        /**
         * Dumps the json not in the right format exception
         * @param array $json The json array
         * @static
         * @access protected
         * @return void
         */
        protected static function dumpBadJson($json){
            ob_start();
            var_dump($json);
            $content = ob_get_contents();
            ob_end_clean();
            throw new \Exception('Error creating the sql: json not in the right format. The json is: ' . $content);            
        }

        /**
         * Constructs a SQL select from a json object
         * For the description of the json format, see the json2sql function
         * @param array $json The json array
         * @static
         * @access protected
         * @return string
         */
        protected static function json2sql_select($json){
            $result = "select SQL_CALC_FOUND_ROWS";
            $fields = "";
            if (array_key_exists('fields', $json)){
                foreach ($json['fields'] as $key => $field){
                    $fields .= "\n\t" . ($fields == '' ? "" : ", ") . $field;
                    if (!is_numeric($key)){
                        $fields .= " as " . $key;
                    }
                }

                $result .= $fields;
            }
            
            $result .= "\nfrom\n" . $json['tables'];

            if (array_key_exists('where', $json)){
                $result .= "\nwhere\n" . $json['where'];
                $user_input = self::getJsonUserInputPart($json, 'where');
                foreach ($user_input as $part){
                    $result .= "\n" . $part;
                }
            }

            if (array_key_exists('having', $json)){
                $result .= "\nhaving\n" . $json['having'];
            }

            if (array_key_exists('grouping', $json)){
                $result .= "\ngroup by\n";
                $grouping = "";
                foreach ($json['grouping'] as $field){
                    $grouping .= "\n\t" . ($grouping == "" ? "" : ", ") . $field;
                }
                $result .= $grouping;
            
            }

            if (array_key_exists('order', $json)){
                $result .= "\norder by";
                $sort = '';
                foreach ($json['order'] as $field => $dir){
                    $sort .= "\n\t" . ($sort == "" ? '' : ', ') . $field . " " . $dir;
                }

                $result .= $sort;
            }

            if (array_key_exists('limit', $json)){
                if (is_array($json['limit'])){
                    if (count($json['limit']) == 1){
                        $result .= "\nlimit " . $json['limit'][0];                        
                    }
                    else {
                        $result .= "\nlimit " . $json['limit'][0] . ", " . $json['limit'][1];
                    }
                }
            }

            return $result;
        }

        /**
         * Process the fields part of an SQL update string
         * @param array $fields The fields part of json
         * @static
         * @access protected
         * @return string
         */
        protected static function json_process_fields_update($fields){
            $result = '';
            foreach ($fields as $field => $value){
                $result .= "\n\t" . ($result == '' ? "" : ", ") . $field . ' = ' . $value ;
            }

            return $result;
        }

        /**
         * Process the fields part of an SQL insert string
         * @param array $fields The fields part of json
         * @static
         * @access protected
         * @return string
         */
        protected static function json_process_fields_insert($fields){
            $result = '';
            foreach ($fields as $field){
                $result .= "\n\t" . ($result == '' ? "" : ", ") . $field;
            }

            return $result;
        }

        /**
         * Constructs a SQL update from a json object
         * For the description of the json format, see the json2sql function
         * @param array $json The json array
         * @static
         * @access protected
         * @return string
         */
        protected static function json2sql_update($json){
            $result = "update";
            $fields = "";

            $result .= "\n" . $json['tables'] . "\nset\n";

            if (array_key_exists('fields', $json)){

                $result .= self::json_process_fields_update($json['fields']);

            }

            $f = self::getJsonUserInputPart($json, 'fields');

            foreach ($f as $fields){
                $result .= self::json_process_fields_update($fields);
            }

            
            if (array_key_exists('where', $json)){
                $result .= "\nwhere\n" . $json['where'];
                $user_input = self::getJsonUserInputPart($json, 'where');
                foreach ($user_input as $part){
                    $result .= "\n" . $part;
                }
            }

            return $result;
        }

        /**
         * Constructs a SQL insert from a json object
         * For the description of the json format, see the json2sql function
         * @param array $json The json array
         * @static
         * @access protected
         * @return string
         */
        protected static function json2sql_insert($json){
            $result = $json['type'] . " into";

            $result .= "\n" . $json['tables'] . "\n";

            if (array_key_exists('fields', $json)){
                $result .= "\n(" . self::json_process_fields_insert($json['fields']) . ")";
            }

            $result .= "\nvalues";

            if (array_key_exists('values', $json)){
                $result .= "\n(" . self::json_process_fields_insert($json['values']) . ")";
            }

            return $result;
        }

        /**
         * Constructs a SQL delete from a json object
         * For the description of the json format, see the json2sql function
         * @param array $json The json array
         * @static
         * @access protected
         * @return string
         */
        protected static function json2sql_delete($json){
            $result = "delete from ";

            $result .= $json['tables'];

            if (array_key_exists('where', $json)){
                $result .= "\nwhere " . $json['where'];
            }

            return $result;
        }

        /**
         * Constructs an SQL query from a json object
         * The json array should have the format:
         * {
         *  'type': 'string (select | insert | update | delete | replace)', 
         *  'fields': [
         *              {'as_part' => 'field_name'}, ... // For the selects OR
         *              {'field_name' => 'value'}, ... // For the updates
         *            ], 
         *  'tables': 'string representing the from part of the select', 
         *  'where': 'string representing the where part of the select [OPTIONAL]', 
         *  'having': 'string representing the having part of the select [OPTIONAL]', 
         *  'grouping': [
         *                'field', ..., '[OPTIONAL]'
         *              ], 
         *  'order': [
         *             {'field_name' => 'asc|desc'}, ... //[OPTIONAL]
         *           ],
         *  'limit' : [from, count] //[OPTIONAL],
         *  'user_input': see the user input function [OPTIONAL]
         * }
         * @param array $json The json array
         * @access public
         * @static
         * @return string
         */
        public static function json2sql($json){
            if (!array_key_exists('type', $json)){
                throw new \Exception('Error creating the sql: json type not specified');
            }
            if (!array_key_exists('fields', $json) && !array_key_exists('tables', $json)){
                if (!array_key_exists('user_input', $json)){
                    self::dumpBadJson($json);                    
                }
            }

            if ($json['type'] == 'select'){
                $result = self::json2sql_select($json);
            }
            else if ($json['type'] == 'update'){
                $result = self::json2sql_update($json);
            }
            else if ($json['type'] == 'insert' || $json['type'] == 'replace'){
                $result = self::json2sql_insert($json);
            }
            else if ($json['type'] == 'delete'){
                $result = self::json2sql_delete($json);
            }

            $pattern = '/\?([^\?]+)\?/';
            if (preg_match_all($pattern, $result, $matches)){
                if (count($matches) >= 1){
                    foreach ($matches[1] as $key){
                        $val = array_key_exists($key, $_GET) ? $_GET[$key] : '';
                        if (substr($key, 0, 2) == '__'){
                            $val = str_replace("'", "\\'", $val);
                        }
                        $result = str_replace('?' . $key . '?', $val, $result );
                    }
                }
            }

            return $result;
        }

        /**
         * Returns the rows for a select
         * @access public
         * @return array
         */
        public function select(){
            $json = json_decode(file_get_contents($this->db_file), true);
            $sql = self::json2sql($json);

            $query = $this->db->mQuery($sql);
            $result = array();
            while ($row = $query->fetch_assoc()){
                $result[] = $row;
            }

            $r = array("items" => $result);

            if (array_key_exists('label', $json)){
                $r['label'] = $json['label'];
            }

            if (array_key_exists('identifier', $json)){
                $r['identifier'] = $json['identifier'];
            }

            return $r;
        }

        /**
         * Does an update
         * @access public
         * @return array
         */
        public function update(){
            $json = json_decode(file_get_contents($this->db_file), true);
            $sql = self::json2sql($json);

            $this->db->mQuery($sql);

            $rows = $this->db->affected_rows;

            $this->db->commit();
            
            return array('affected_rows' => $rows);
        }

        /**
         * Does an insert
         * @access public
         * @return array
         */
        public function insert(){
            $json = json_decode(file_get_contents($this->db_file), true);
            $sql = self::json2sql($json);

            $this->db->mQuery($sql);

            $rows = $this->db->affected_rows;
            $id = $this->db->insert_id;

            $this->db->commit();
            
            return array('affected_rows' => $rows, 'id' => $id);
        }

        /**
         * Does an delete
         * @access public
         * @return array
         */
        public function delete(){
            $json = json_decode(file_get_contents($this->db_file), true);
            $sql = self::json2sql($json);

            $this->db->mQuery($sql);

            $rows = $this->db->affected_rows;

            $this->db->commit();
            
            return array('affected_rows' => $rows);
        }
    }
}
?>
