<?php
/**
 * Class used for rendering the list of theme topics
 *
 * @license http://ec.europa.eu/idabc/eupl
 */
namespace tariqa3{

class ThemeTopicsSection extends tariqa3ModuleSection{

	/**
	 * The theme for which we are displaying the topics
	 * @var Theme
	 * @access private
	 */
	private $theme;

	/**
	 * The translation array
	 * @var array
	 * @access public
	 */
	public $translationTable;

	/**
	 * The list of topics for the theme
	 * @var ThemeTopic[]
	 * @access private
	 */
	private $topics;

	/**
	 * Generates a new tab for a topic to be displayed
	 * @param tariqaUser $user The user which is performing the operation
	 * @param int $topic_id The topic identifier
	 * @param int $query_id The query identifier
	 * @param array $translationTable The translation table
	 * @param string $template_path The path to the template files
	 * @access public
	 * @static
	 * @return string
	 */
	public static function generateTab($user, $topic_id, $query_id, $translationTable, $template_path) {
		$topic = statics::getThemeTopic($topic_id, $user->get_access_level());
		/* @var $topic_query TopicQuery */
		/* @var $query TopicQuery */
		$query = null;
		foreach ($topic->queries as $topic_query){
			if ($topic_query->query->query_id == $query_id){
				$query = $topic_query;
				break;
			}
		}

		if ($query == null){
			return '<error>There was a problem fetching the topic query from the database</error>';
		}

		$feed_id = '';
		if ($query->query->engine == null){
			$query->query = statics::getQuery($query_id);
		}
		if ($query->query->engine != null){
			if ($query->query->engine->engine_id == FullAggregators::LATESTNEWS || $query->query->engine->engine_id == FullAggregators::RSS){
				$feed_id = "<feed_id>{$query->query->query}</feed_id>";
			}
		}

		$custom_filters = $user->get_preference('theme_page/' . $topic->theme_id . '/queries/filters');
		$query->query->flavour->flavour_id = $query_id;
		$filtering = NewsSection::getFiltering($query->query->query_id, $query->query, $custom_filters, $translationTable, $template_path);

		$html = rawurlencode($filtering['html']);

		$config = \MetaNull\Tools\mConfig::loadConfiguration(APPLICATION_MAIN_CONFIG_FILE, APPLICATION_CONFIG_FORMAT, 'app_config');
		$tpl2 = new \MetaNull\Application\MTemplate('m');
		$tpl2->loadTemplate($template_path . '/tab_loading.html');
		$tpl2->data['display'] = '';
		$tpl2->data['result_id'] = 'news_' . $query_id;
		$tpl2->data['img_path'] = $config['templates']['baseurl'] . '/' . $config['templates']['default'] . '/images';

		$title = rawurlencode($query->title . ' ' . $tpl2->transform());

		return "<title>{$title}</title><html>{$html}</html><query_id>{$query_id}</query_id>" . $feed_id;
	}

	/**
	 * Constructs the object
	 * @param array $config The configuration array
	 * @param string $parentBase base_folder of the container class
	 * @param string $section_id The section identifier
	 * @param Theme $theme The theme for which we are displaying the topics
	 * @param array $translationTable The translation table
	 * @access public
	 * @return void
	 */
	public function __construct($config, $parentBase, $section_id, $theme, $translationTable) {
		parent::__construct($config, $parentBase, $section_id);
		$this->translationTable = $translationTable;

		$this->theme = $theme;
		$this->topics = statics::ListThemeTopics($this->theme->theme_id, $this->user->get_access_level());
	}

	/**
	 * Render the module
	 * @return string
	 */
	public function getPageContent() {
		$tpl = new \MetaNull\Application\MTemplate( 'm' );
		$tpl->loadTemplate($this->getTemplateFilePath('topics.html'));
		$tpl->data = $this->translationTable;

		$this->info_setJavascript(
			array(
				'scripts/theme_topics.js',
				'scripts/ddsmoothmenu.js',
			)
		);

		$this->info_setStyleSheet(array(
			$this->config['templates']['baseurl'] . '/' . $this->config['templates']['default'] . '/styles/ddsmoothmenu.css',
		));
		
		$newsConfig = \MetaNull\Tools\mConfig::loadConfiguration(APPLICATION_NEWS_CONFIG_FILE, APPLICATION_CONFIG_FORMAT, 'news_config');
		$custom_filters = $this->user->get_preference('theme_page/' . $this->theme->theme_id . '/queries/filters');

		$tpl->data['topics'] = array();
		$tpl->data['result_ids'] = array();

		/* @var $topic ThemeTopic */
		foreach ($this->topics as $topic){
			$subtopics = array();
			/* @var $query TopicQuery */
			foreach ($topic->queries as $query){
				$subtopics[] = array(
					'id' => $query->query->query_id,
					'title' => $query->title, 
				);

				$range = new timeRange($query->query->time_range);

				$qstr = $query->query->query;
				$start_date = null;
				$end_date = null;

				if ($custom_filters != null) if (array_key_exists($query->query->query_id, $custom_filters)){
					$filter = NewsSection::renderCustomFilter($custom_filters[$query->query->query_id], $query->query, $this->translationTable, $this->getTemplatesFolder());
					if ($filter['start_date'] != null) $start_date = $filter['start_date'];
					if ($filter['end_date'] != null) $end_date = $filter['end_date'];
					$qstr = $filter['query'];
				}

				if ($start_date == null || $end_date == null){
					$c = \MetaNull\News\unifiedQuerySyntax::getDatesCondition($qstr);
					if (!array_key_exists('string', $c)){
						if ($start_date == null) $start_date = $range->getStartDate();
						if ($end_date == null) $end_date = $range->getEndDate();
					}
				}

				$tpl->data['result_ids'][] = array(
					'query_id' => $query->query->query_id,
					'text' => 'query=' . rawurlencode($qstr) . '&source_id=' .
							$query->query->source_id . ($start_date != null?'&start_date=' . $start_date:'') .
							($end_date != null?'&end_date=' . $end_date:'') .
							'&count=' . $newsConfig['search']['default']['results-per-page'],
				);
			}

			$tpl->data['topics'][] = array(
				'title' => $topic->title,
				'subtopics' => $subtopics,
				'topic_id' => $topic->topic_id, 
			);
		}

		$this->info_setBodyOnLoad('initMenu()');

		return $tpl->transform();
	}
}
}
?>
