<?php
/*
 * © EUROPEAN COMMISSION, 2007-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou implicites.
 * Consultez la Licence pour les autorisations et les restrictions linguistiques spécifiques relevant de la Licence.
 *
 * Auteurs:
 *	Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 *	Cosmin Popescu,   Adrian.POPESCU@ext.ec.europa.eu
*/

namespace tariqa3\search{
/**
 * Base class for fetching the news results from the Factiva engine
 * @author Cosmin Popescu
 *
 */
class FactivaEngine extends searchEngine {

    /**
     * Constructs the object
     * @param array $newsConfig The news configuration array
     * @access public
     * @return void
     */
    public function __construct($newsConfig) {
	parent::__construct($newsConfig);
    }

    /**
     * Fetches a list of articles
     * @param newsQuery $query The query
     * @access public
     * @return tariqaSearchResultSet
     */
    public function listArticles($query) {
	$queryObj = new \FactivaContentQuery();
	$queryObj->searchString = $query->query;
	if ($query->first != null) $queryObj->first = $query->first;
	if ($query->count != null) $queryObj->count = $query->count;
	if ($query->source_id != null) {
	    $source = \tariqa3\statics::getSource($query->source_id);
	    if ($source->source_data != null) {
		$parts = explode(";", $source->source_data);
		$sources_arr = array();
		foreach ($parts as $s) $sources_arr[] = $s;
		$queryObj->sourceCodes = $sources_arr;
	    }
	}
	$results = \tariqa3\statics::$tariqaClient->FactivaContentSearch(\tariqa3\statics::$tariqaToken, $queryObj);

	$result = new tariqaSearchResultSet();

	$result->query = $query->query;
	$engine = \tariqa3\statics::getEngine($query->engine_id);
	$result->engine_name = $engine->name;
	$result->engine = $query->engine_id;
	$result->source = $query->source_id;
	$result->from = $query->first;
	$result->count = $results->queryHitCount;
	$result->results_per_page = $query->count !== null?$query->count:count($results->results);

	$result->documents = array();

	/* @var $article FactivaContentHeadline */
	foreach ($results->results as $article) {
	    $document = new tariqaDocument();

	    $title_str = "";
	    foreach ($article->headline as $part) $title_str .= ($title_str == ""?"":" ") . $part;

	    $document->title = $title_str;
	    $document->id = $query->engine_id . ':' .
		    $query->source_id . ':' .
		    urlencode($article->accessionNo);
	    $document->url = $this->link('Factiva:' . $query->source_id . ':' . urlencode($article->accessionNo));
	    $document->js_link = $this->js_link(
		    'Factiva:' . $query->source_id . ':' . urlencode($article->accessionNo),
		    $title_str
	    );

	    $document->date = $article->publicationDate;

	    $snippet = "";
	    foreach ($article->snippet as $part) $snippet .= ($snippet == ""?"":"\n") . $part;

	    $document->summary = $snippet;
	    $document->publisher = $article->sourceName;

	    $result->documents[] = $document;
	}

	return $result;
    }

    /**
     * Gets a specific article
     * @param string $article_id The article identifier
     * @access public
     * @abstract
     * @return tariqaFullDocument
     */
    public function getArticle($article_id) {
	$parts = explode(':', $article_id, 3);
	if (count($parts) != 3) {
	    throw new \Exception('Document identifier not well formed');
	}

	$document = \tariqa3\statics::$tariqaClient->FactivaGetDocument(\tariqa3\statics::$tariqaToken, $id);

	$result = new tariqaFullDocument();
	$result->id = $article_id;
	$result->engine = $parts[0];
	$result->source = $parts[1];
	$result->document_id = $document->accessionNo;
	$title_str = '';
	foreach ($document->headline as $part) $title_str .= ($title_str == ""?"":" ") . $part;
	$result->title = $title_str;

	$result->date = $document->publicationDate;
	$result->publisher = $document->sourceName;

	$body = '';
	foreach ($document->leadParagraph as $paragraph) $body .= ($body == ''?'':'<br/>') . $paragraph;
	foreach ($document->tailParagraph as $paragraph) $body .= ($body == ''?'':'<br/>') . $paragraph;
	$result->body = $body;

	return $result;
    }

    /**
     * Gets the multi ajax search object for any page but the search page. For example country pages, home page, news wires page etc.
     * @param Flavour $flavour The flavour
     * @param array $filtering The filtering data
     * @param Query $flavour_query The flavour current query
     * @access public
     * @return multiAjaxSearch
     */
    public function newsSection($flavour, $filtering, $flavour_query) {
	$app_config = \MetaNull\Tools\mConfig::loadConfiguration(APPLICATION_MAIN_CONFIG_FILE, APPLICATION_CONFIG_FORMAT, 'app_config');
	$search = new \tariqa3\multiAjaxSearch($filtering['query'], $flavour_query->source_id, $app_config['application']['service_url'],
	    'search_' . $flavour->flavour_id, true, $filtering['start_date'], $filtering['end_date'], 0, $this->newsConfig['search']['default']['results-per-page'],
	    \tariqa3\multiAjaxSearch::ORIGINAL_SYNTAX);

	$search->noNewTab();
	$search->setNavigationId('navigation_' . $flavour->flavour_id);
	$search->setResultId('news_' . $flavour->flavour_id);

	return $search;
    }

    /**
     * Gets the multi ajax search object
     * @param string $query The query string
     * @param int $from The first result to display
     * @param int $count The number of results to display
     * @param DateTime $startDate The start date [OPTIONAL]
     * @param DateTime $endDate The end date [OPTIONAL]
     * @access public
     * @abstract
     * @return multiAjaxSearch
     */
    public abstract function multiAjaxSearch($query, $from, $count, $startDate = null, $endDate = null){
	return null;
    }
}
}
?>