<?php

/*
 * © EUROPEAN COMMISSION, 2007-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou implicites.
 * Consultez la Licence pour les autorisations et les restrictions linguistiques spécifiques relevant de la Licence.
 *
 * Auteurs:
 * 	Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 * 	Cosmin Popescu,   Adrian.POPESCU@ext.ec.europa.eu
*/

namespace tariqa3\search {

/**
 * Base class for fetching the news results from the Latestnews engine
 * @author Cosmin Popescu
 *
 */
class LatestNewsEngine extends searchEngine {

    /**
     * Constructs the object
     * @param array $newsConfig The news configuration array
     * @access public
     * @return void
     */
    public function __construct($newsConfig) {
	parent::__construct($newsConfig);
    }

    /**
     * Fetches a list of articles
     * @param newsQuery $query The query
     * @access public
     * @return tariqaSearchResultSet
     */
    public function listArticles($query) {
	$start_date = null;
	$end_date = null;

	if ($query->start_date != null) {
	    $start_date = $this->date2range($query->start_date);
	}
	if ($query->end_date != null) {
	    $end_date = $start_date - $this->date2range($query->end_date) + 1;
	}

	/* if ($end_date > $start_date) {
              $end_date = $start_date;
              } */

	//$query->query = str_replace("\\", '', $query->query);
	if ($query->count == null) {
	    $query->count = $this->newsConfig['default']['results-per-page'];
	}

	if ($query->first == 0) {
	    $query->first = 1;
	}

	if ($query->specific_data != null){
	    $data = \tariqa3\statics::post2array($query->specific_data);
	    foreach ($data as $key => $value){
		if ($key == 'languages'){
		    $lang = explode(';', $value);
		    $s = '';
		    foreach ($lang as $l){
			$s .= ($s == ''?'':' or ') . "language = '{$l}'";
		    }

		    if ($s != ''){
			$query->query .= " and ({$s})";
		    }
		}
	    }
	}

	$results = \tariqa3\statics::$tariqaClient->fetchLatestNews(
		\tariqa3\statics::$tariqaToken,
		$query->query,
		$start_date,
		($query->first !== null ? floor(($query->first - 1) / $query->count) : null),
		($query->first !== null ? $query->count : null), true, $end_date
	);

	$result = new tariqaSearchResultSet();

	$result->query = $query->query;
	$engine = \tariqa3\statics::getEngine($query->engine_id);
	$result->engine_name = $engine->name;
	$result->engine = $query->engine_id;
	$result->source = $query->source_id;
	//\MetaNull\Tools\mDump::dumpVariable($query->query_name);
	$result->from = $query->first;

	$result->logo = $engine->logo_url;
	$result->provider_url = $engine->doc_url;
	$result->count = $results->available_articles;
	$result->results_per_page = $query->count !== null ? $query->count : count($results->articles);

	$data = \tariqa3\statics::post2array(\MetaNull\Tools\mTools::rawurldecode($query->specific_data));

	if (array_key_exists('source_name', $data)) {
	    $result->source_name = $data['source_name'];
	}

	$result->documents = array();

	/* @var $article Article */
	foreach ($results->articles as $article) {
	    $document = new tariqaDocument();

	    $document->title = $article->title;
	    $document->id = $query->engine_id . ':' . $query->source_id . ':' . urlencode($article->id);
	    $document->url = $this->link('LatestNews::' . urlencode($article->id));
	    $document->js_link = $this->js_link('LatestNews::' . urlencode($article->id), $article->title);

	    $d = preg_replace('/^([^\-]*)\-([^\-]*)\-([^T]*)T([^:]*):([^:]*):([^\+\-]*)[\+|\-].*$/', '\1-\2-\3 \4:\5:\6', $article->date);
	    $d = new \DateTime($d);

	    $document->date = $d->getTimestamp();

	    $document->summary = $article->lead_para;
	    $document->publisher = '';

	    $result->documents[] = $document;
	}

	return $result;
    }

    /**
     * Gets a specific article
     * @param string $article_id The article identifier
     * @access public
     * @abstract
     * @return tariqaFullDocument
     */
    public function getArticle($article_id) {
	$parts = explode(':', $article_id, 3);
	if (count($parts) != 3) {
	    throw new \Exception('Document identifier not well formed');
	}

	$document = \tariqa3\statics::$tariqaClient->latestNewsGetDocument(\tariqa3\statics::$tariqaToken, $parts[2]);

	$result = new tariqaFullDocument();
	$result->id = $article_id;
	$result->engine = $parts[0];
	$result->source = $parts[1];
	$result->document_id = $document->id;
	$result->title = $document->headline;

	$result->date = $document->date;
	$result->publisher = $document->agency;
	$body = $document->text;

	$body = implode('</p><p>', preg_split('/^[ \t]{2,}/m', $body));
	$body = '<p>' . $body . '</p>';
	
	if (\tariqa3\statics::_isIE()){
	    $result->body = '<pre style="word-wrap: break-word; font-family: Verdana, Arial">' . $body . "</pre>";
	}
	else {
	    $result->body = '<span style="white-space: pre-wrap;">' . $body . "</span>";
	}

	return $result;
    }

    /**
     * Gets the multi ajax search object for any page but the search page. For example country pages, home page, news wires page etc.
     * @param Flavour $flavour The flavour
     * @param array $filtering The filtering data
     * @param Query $flavour_query The flavour current query
     * @access public
     * @return multiAjaxSearch
     */
    public function newsSection($flavour, $filtering, $flavour_query) {
	$app_config = \MetaNull\Tools\mConfig::loadConfiguration(APPLICATION_MAIN_CONFIG_FILE, APPLICATION_CONFIG_FORMAT, 'app_config');
	$search = new \tariqa3\multiAjaxSearch($filtering['query'], $flavour_query->source_id, $app_config['application']['service_url'],
	    'search_' . $flavour->flavour_id, true, $filtering['start_date'], $filtering['end_date'], 0, $this->newsConfig['search']['default']['results-per-page'],
	    \tariqa3\multiAjaxSearch::ORIGINAL_SYNTAX);

	$specific_data = "source_name=" . $flavour_query->title;
	$search->setSpecificData(rawurlencode($specific_data));

	$search->noNewTab();
	$search->setNavigationId('navigation_' . $flavour->flavour_id);
	$search->setResultId('news_' . $flavour->flavour_id);

	return $search;
    }

    /**
     * Gets the multi ajax search object
     * @param string $query The query string
     * @param int $from The first result to display
     * @param int $count The number of results to display
     * @param DateTime $startDate The start date [OPTIONAL]
     * @param DateTime $endDate The end date [OPTIONAL]
     * @access public
     * @return multiAjaxSearch
     */
    public function multiAjaxSearch($query, $from, $count, $startDate = null, $endDate = null){
	$app_config = \MetaNull\Tools\mConfig::loadConfiguration(APPLICATION_MAIN_CONFIG_FILE, APPLICATION_CONFIG_FORMAT, 'app_config');
	$defaults = $this->newsConfig['default'];
	$search = new \tariqa3\multiAjaxSearch($query, $defaults['latestnews']['source_id'], $app_config['application']['service_url'],
	    'searchResult', false, ($startDate == null?null:$startDate->getTimestamp() + $startDate->getOffset()),
	    ($endDate == null?null:$endDate->getTimestamp() + $endDate->getOffset()), $from, $count, \tariqa3\multiAjaxSearch::ORIGINAL_SYNTAX);

	$languages = \tariqa3\statics::LatestNewsListLanguages();
	$set = false;
	$l = '';
	foreach ($languages as $language){
	    if (!array_key_exists('ln_lang_' . $language->id, $_POST) && !array_key_exists('ln_lang_' . $language->id, $_GET)){
		$set = true;
	    }
	    else {
		if (array_key_exists('ln_lang_' . $language->id, $_POST)){
		    $l .= ($l == ''?'':';') . $_POST['ln_lang_' . $language->id];
		}
		else {
		    $l .= ($l == ''?'':';') . $_GET['ln_lang_' . $language->id];
		}
	    }
	}

	if ($set){
	    $search->setSpecificData('languages=' . $l);
	}

	return $search;
    }

    /**
     * Gets the POST data used for saving a search in the briefcase
     * @param string $query_txt The query string text
     * @param Query $query The query
     * @param int $start_date The start date (unix timestamp) [OPTIONAL]
     * @param int $end_date The end date (unix timestamp) [OPTIONAL]
     * @param array $additional_parameters Additional parameters [OPTIONAL]
     * @access public
     * @return string
     */
    public function getSavePost($query_txt, $query, $start_date = null, $end_date = null, $additional_parameters = array()){
	$range = '';
	$range_days = '';
	if ($start_date != null) $range = floor((time() - $start_date) / (60 * 60 * 24));
	if ($end_date != null) $range_days = $range - floor((time() - $end_date) / (60 * 60 * 24));
	if ($range_days > $range){
	    $range = $range_days;
	}
	$post_data = "ln_date_type=range&selected=1&engine=LatestNews&query={$query_txt}&ln_range={$range}&range_days={$range_days}" .
		"&range_field_name=ln_range&date_type_field_name=ln_date_type" .
		"&ln_start_date=&ln_end_date=&from_field_name=ln_start_date&to_field_name=ln_end_date&results_per_page={$this->newsConfig['search']['default']['results-per-page']}";

	return $post_data;
    }

}

}
?>