<?php
/*
 * © EUROPEAN COMMISSION, 2007-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou implicites.
 * Consultez la Licence pour les autorisations et les restrictions linguistiques spécifiques relevant de la Licence.
 *
 * Auteurs:
 *	Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 *	Cosmin Popescu,   Adrian.POPESCU@ext.ec.europa.eu
*/

namespace tariqa3\search{
/**
 * Base class for fetching the news results from the OxfordAnalytica engine
 * @author Cosmin Popescu
 *
 */
class OxfordAnalyticaEngine extends searchEngine {

    /**
     * The Oxford Analytica URL
     * @var string
     */
    const OXAN_URL = 'http://www.oxan.com';

    /**
     * Constructs the object
     * @param array $newsConfig The news configuration array
     * @access public
     * @return void
     */
    public function __construct($newsConfig) {
	parent::__construct($newsConfig);
    }

    /**
     * Fetches a list of articles
     * @param newsQuery $query The query
     * @access public
     * @return tariqaSearchResultSet
     */
    public function listArticles($query) {
	$queryObj = new \OxfordAnalyticaDetailedQuery();
	$queryObj->freetext = $query->query;
	$queryObj->primary = false;
	$queryObj->storyType = \OxfordAnalyticaQueryBase::ST_FULL_LENGTH_ARTICLE;

	if ($query->specific_data != '' && $query->specific_data != null) {
	    $data = \tariqa3\statics::post2array(\MetaNull\Tools\mTools::rawurldecode($query->specific_data));
	    if (array_key_exists('type', $data)) {
		$queryObj->storyType = $data['type'];
	    }
	    if (array_key_exists('country', $data)) {
		$queryObj->country = $data['country'];
		$queryObj->primary = true;
	    }
	    if (array_key_exists('titling', $data)) {
		$queryObj->titling = $data['titling'];
	    }
	    if (array_key_exists('topic', $data)) {
		$queryObj->topic = $data['topic'];
		$queryObj->primary = true;
	    }
	}

	if ($query->start_date != null) {
	    $queryObj->from = $query->start_date;
	}
	if ($query->end_date != null) {
	    $queryObj->to = $query->end_date;
	}

	$queryObj->pageLen = ($query->count != null ? $query->count : 20);

	if ($query->first !== null) {
	    $queryObj->pageNo = floor($query->first / $queryObj->pageLen);
	}

	$queryObj->displayMode = \OxfordAnalyticaQueryBase::DM_HTML;

	$results = \tariqa3\statics::$tariqaClient->oxanDetailedSearch(\tariqa3\statics::$tariqaToken, $queryObj);

	$result = new tariqaSearchResultSet();

	$result->query = $query->query;
	$engine = \tariqa3\statics::getEngine($query->engine_id);
	$result->engine_name = $engine->name;
	$result->logo = $engine->logo_url;
	$result->engine = $query->engine_id;
	$result->disclaimer = str_replace(array("\n", "\r"), '', $engine->disclaimer);
	$result->source = $query->source_id;
	$result->from = $query->first;

	$result->count = $results->totalNumberOfMatches;
	$result->results_per_page = $query->count !== null?$query->count:count($results->hitItem);

	$result->documents = array();

	/* @var \OxfordAnalyticaHitItem $article */
	foreach ($results->hitItem as $article) {
	    $document = new tariqaDocument();

	    $title_str = $article->headline;

	    $document->title = $title_str;
	    $document->id = $query->engine_id . ':' .
		    $query->source_id . ':' .
		    $article->itemId;
	    $document->url = $this->link('OxfordAnalytica:' . $query->source_id . ':' . urlencode($article->itemId));
	    $document->js_link = $this->js_link(
		    'OxfordAnalytica:' . $query->source_id . ':' . urlencode($article->itemId),
		    $title_str
	    );

	    $document->date = $article->firstRunDate;

	    $document->summary = "";
	    $document->publisher = "";

	    $result->documents[] = $document;
	}

	return $result;
    }

    /**
     * Gets a specific article
     * @param string $article_id The article identifier
     * @access public
     * @abstract
     * @return tariqaFullDocument
     */
    public function getArticle($article_id) {
	$parts = explode(':', $article_id, 3);
	if (count($parts) != 3) {
	    throw new \Exception('Document identifier not well formed');
	}

	if (preg_match('/^DB[\d]+$/', $parts[2])){
	    $page = '/display.aspx?ItemID=' . $parts[2];
	}
	else {
	    if (strpos($parts[2], '?') === false){
		$page = '/display.aspx?' . $parts[2];
	    }
	    else {
		$page = str_replace('?', '&', $parts[2]);
	    }
	}

	$oxandoc = \tariqa3\statics::$tariqaClient->oxanFetchUrl(\tariqa3\statics::$tariqaToken, $page);

	$options = array(
		"drop-proprietary-attributes" => true,
		"doctype" => "omit",
		"clean" => true,
		"output-xhtml" => true,
	);
	$tidy = tidy_parse_string( $oxandoc->body, $options, "utf8" );
	$tidy->cleanRepair();
	$body = (string)$tidy;
	$body = \preg_replace('/xmlns="[^"]*"/', "", $body);
	$doc = new \DOMDocument('1.0', 'UTF-8');
	$doc->loadXML($body);

	$result = new tariqaFullDocument();
	$result->id = $article_id;
	$result->engine = $parts[0];
	$result->source = $parts[1];
	$result->document_id = $parts[2];

	$result->date = strtotime($oxandoc->date);
	$result->publisher = null;
	$result->body = $oxandoc->body;

	$result->body = preg_replace('/<img([^s|s^r|sr^c]*)src[ |\t|\n|\r]*=[ |\t|\n|\r]*"([^"]*)"([^>]*)>/', '<img\1src="' . self::OXAN_URL . '\2"\3>', $result->body);
	//$result->body = preg_replace('/<a([^h|h^r|hr^e|hre^f]*)href[ |\t|\n|\r]*=[ |\t|\n|\r]*"([^"]*)\.jpg"([^>]*)>/', '<a\1href="' . self::OXAN_URL . '\2.jpg"\3>', $result->body);
	$result->body = '<p style="text-align: right"><a href="#follow_up_analyst">Follow up Oxford Analytica with an analyst</a></p>' . $result->body;
	//$result->body = preg_replace('/<a([^h|h^r|hr^e|hre^f]*)href[ |\t|\n|\r]*=[ |\t|\n|\r]*"([^"]*)"([^>]*)>/', '<a\1href="' . self::OXAN_URL . '\2"\3>', $result->body);

	$result->title = $oxandoc->headline;

	if ($oxandoc->headline == '' || $oxandoc->headline == null) {
	    $xp = new \DOMXPath($doc);
	    $rs = $xp->query("//p[@class='subject']");
	    if ($rs->length > 0) {
		$result->title = $rs->item(0)->nodeValue;
	    }
	}

	$result->body .= '<hr/><p><a href="mailto:client.services@oxford-analytica.com?subject=Follow up - ' . \MetaNull\Tools\mTools::rawurlencode($result->title) . '" id="follow_up_analyst">Follow up with an Oxford Analytica analyst</a><br/><br/>Oxford Analytica will answer client requests for amplification or correction within one business day.<br/>Call Oxford Analytica Client Services on +44 1865 261 600, or email <a href="mailto:client.services@oxford-analytica.com?subject=Follow up - ' . \MetaNull\Tools\mTools::rawurlencode($result->title) . '">client.services@oxford-analytica.com</a></p>';
	//\MetaNull\Tools\mDump::dumpVariable($result);

	return $result;
    }

    /**
     * Gets the multi ajax search object for any page but the search page. For example country pages, home page, news wires page etc.
     * @param Flavour $flavour The flavour
     * @param array $filtering The filtering data
     * @param Query $flavour_query The flavour current query
     * @access public
     * @return multiAjaxSearch
     */
    public function newsSection($flavour, $filtering, $flavour_query) {
	$app_config = \MetaNull\Tools\mConfig::loadConfiguration(APPLICATION_MAIN_CONFIG_FILE, APPLICATION_CONFIG_FORMAT, 'app_config');
	$search = new \tariqa3\multiAjaxSearch($filtering['query'], $flavour_query->source_id, $app_config['application']['service_url'],
	    'search_' . $flavour->flavour_id, true, $filtering['start_date'], $filtering['end_date'], 0, $this->newsConfig['search']['default']['results-per-page'],
	    \tariqa3\multiAjaxSearch::ORIGINAL_SYNTAX);

	$search->noNewTab();
	$search->setNavigationId('navigation_' . $flavour->flavour_id);
	$search->setResultId('news_' . $flavour->flavour_id);

	return $search;
    }

    /**
     * Gets the multi ajax search object
     * @param string $query The query string
     * @param int $from The first result to display
     * @param int $count The number of results to display
     * @param DateTime $startDate The start date [OPTIONAL]
     * @param DateTime $endDate The end date [OPTIONAL]
     * @access public
     * @return multiAjaxSearch
     */
    public function multiAjaxSearch($query, $from, $count, $startDate = null, $endDate = null){
	$app_config = \MetaNull\Tools\mConfig::loadConfiguration(APPLICATION_MAIN_CONFIG_FILE, APPLICATION_CONFIG_FORMAT, 'app_config');
	$defaults = $this->newsConfig['default'];
	$search = new \tariqa3\multiAjaxSearch($query, $defaults[$_POST['engine']]['source_id'], $app_config['application']['service_url'],
	    'searchResult', false, ($startDate == null?null:$startDate->getTimestamp() + $startDate->getOffset()),
	    ($endDate == null?null:$endDate->getTimestamp() + $endDate->getOffset()), $from, $count, \tariqa3\multiAjaxSearch::ORIGINAL_SYNTAX);
	$data = 'type=' . ($_POST['oxan_search_type'] == \tariqa3\FullOxfordAnalytica::FULL_TYPE_SEARCH ? \OxfordAnalyticaQueryBase::ST_FULL_LENGTH_ARTICLE : \OxfordAnalyticaQueryBase::ST_EXECUTIVE_SUMMARY);

	if ($_POST['selected_country'] != ''){
	    $data .= '&country=' . $_POST['selected_country'];
	}
	if ($_POST['selected_topic'] != ''){
	    $data .= '&topic=' . $_POST['selected_topic'];
	}
	if ($_POST['oxan_titling'] != ''){
	    $data .= '&titling=' . $_POST['oxan_titling'];
	}
	$search->setSpecificData(rawurlencode($data));

	return $search;
    }

    /**
     * Gets the POST data used for saving a search in the briefcase
     * @param string $query_txt The query string text
     * @param Query $query The query
     * @param int $start_date The start date (unix timestamp) [OPTIONAL]
     * @param int $end_date The end date (unix timestamp) [OPTIONAL]
     * @param array $additional_parameters Additional parameters [OPTIONAL]
     * @access public
     * @return string
     */
    public function getSavePost($query_txt, $query, $start_date = null, $end_date = null, $additional_parameters = array()){
	return '';
    }
}
}
?>