<?php
/*
 * © EUROPEAN COMMISSION, 2007-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou implicites.
 * Consultez la Licence pour les autorisations et les restrictions linguistiques spécifiques relevant de la Licence.
 *
 * Auteurs:
 *	Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 *	Cosmin Popescu,   Adrian.POPESCU@ext.ec.europa.eu
*/

namespace tariqa3\search{
/**
 * Base class for fetching the news results from an engine
 * @author Cosmin Popescu
 *
 */
abstract class searchEngine {

    /**
     * The news configuration
     * @var array
     * @access protected
     */
    protected $newsConfig;

    /**
     * The date format
     * @var string
     * @access protected
     */
    protected $date_format;

    /**
     * Constructs the object
     * @param array $newsConfig The news configuration array
     * @access public
     * @return void
     */
    public function __construct($newsConfig) {
	if (array_key_exists('search', $newsConfig)) {
	    $this->newsConfig = $newsConfig['search'];
	}
	else {
	    $this->newsConfig = $newsConfig;
	}
    }

    /**
     * Adds a node to the result xml
     * @param string $name The node name
     * @param string $value The value of the node
     * @param $doc DOMDocument The XML document
     * @param boolean $cdata If true, then the node is added as a CDATA section [OPTIONAL]
     * @param string $parameters The node parameters [OPTIONAL]
     * @access public
     * @static
     * @return \DOMElement
     */
    public static function addNode($name, $value, $doc, $cdata = false, $parameters = null) {
	$element = $doc->createElement($name);
	if ($parameters != null) {
	    $param = \tariqa3\statics::post2array($parameters);
	    foreach ($param as $key => $value) {
		$element->setAttribute($key, $value);
	    }
	}
	if (!$cdata) {
	    $element->nodeValue = $value;
	}
	else {
	    $data = $doc->createCDATASection($value);
	    $element->appendChild($data);
	}

	return $element;
    }

    /**
     * Transforms the additional controls from array to string
     * @param array $controls The controls
     * @access protected
     * @static
     * @return string
     */
    protected static function controls2string($controls){
        $result = '';
        foreach($controls as $key => $value){
            $result .= '<li><a href="#" onclick="' . $value . '; return false;">' . $key . '</a>';
        }

        return $result;
    }

    /**
     * Converts a tariqaSearchResultSet or a tariqaFullDocument to a DOM XML
     * @param tariqaSearchResultSet|tariqaFullDocument $result The result set
     * @param array $config The news configuration array
     * @access public
     * @static
     * @return DOMDocument
     */
    public static function transformToXML($result, $config) {
	if (array_key_exists('search', $config)) {
	    $newsConfig = $config['search'];
	}
	else {
	    $newsConfig = $config;
	}

	$date_format = null;
	if (array_key_exists("dates", $newsConfig)) {
	    if (array_key_exists("format", $newsConfig["dates"])) {
		$date_format = $newsConfig["dates"]["format"];
	    }
	}

	$doc = new \DOMDocument('1.0', 'UTF-8');

	if ($result instanceof tariqaSearchResultSet) {
	    if ($result === null) {
		throw new \Exception('Could not fetch the rows from the SOAP server');
	    }
	    $root = $doc->createElement('articles');

	    $root->appendChild(self::addNode('query', $result->query, $doc, true));
	    $root->appendChild(self::addNode('engine', $result->engine, $doc));
	    $root->appendChild(self::addNode('engine_name', $result->engine_name, $doc));
	    if ($result->source_name != null) {
		$root->appendChild(self::addNode('source_name', $result->source_name, $doc, true));
	    }
	    else {
		if ($result->source != null) {
		    $s = \tariqa3\statics::getSource($result->source);
		    $root->appendChild(self::addNode('source_name', $s->title, $doc, true));
		}
		else $root->appendChild(self::addNode('source_name', '', $doc, true));
	    }
	    $root->appendChild(self::addNode('source', ($result->source != null?$result->source:''), $doc));
	    if ($result->disclaimer != null) {
		$root->appendChild(self::addNode('disclaimer', $result->disclaimer, $doc));
	    }
	    $root->appendChild(self::addNode('logo', $result->logo, $doc));
	    $root->appendChild(self::addNode('provider_url', $result->provider_url, $doc));
	    $root->appendChild(self::addNode('from', ($result->from !== null?$result->from:1), $doc));
	    $root->appendChild(self::addNode('count', $result->count, $doc, false, 'value=' . $result->count));
	    $root->appendChild(self::addNode('results_per_page', $result->results_per_page, $doc));
	    $root->appendChild(self::addNode('additional_js', ($result->js_code != null?$result->js_code:''), $doc, true));

	    $n = $result->from;

	    $fakedoc = new \DOMDocument('1.0', 'UTF-8');
	    $html_ids = $fakedoc->createElement("div");
	    $html_ids->setAttribute("style", "display: none");

	    $result_ids = $fakedoc->createElement("xtariqa:result-ids");

	    /* @var $document tariqaDocument */
	    foreach ($result->documents as $document) {
		$xml_article = $doc->createElement("article");
		$xml_article->appendChild(self::addNode('title', $document->title, $doc, true));
		$xml_article->appendChild(self::addNode('idx', $n++, $doc, true));
		$xml_article->appendChild(self::addNode('id', $document->id, $doc, true));
		$xml_article->appendChild(self::addNode('url', $document->url, $doc, true));
		$xml_article->appendChild(self::addNode('js_link', $document->js_link, $doc, true));
		if ($document->date == 0 || $document->date == null) $d = $newsConfig['dates']['no-date'];
		else {
		    if ($date_format != null) $d = date($date_format, $document->date);
		}
		$xml_article->appendChild(self::addNode('date', $d, $doc, true));
		$xml_article->appendChild(self::addNode('summary', $document->summary, $doc, true));
		$xml_article->appendChild(self::addNode('source', $document->publisher, $doc, true));

		if (is_numeric($document->date)) {
		    $d = new \DateTime(date('Y-m-d H:i:s', $document->date));
		    $diff = $d->diff(new \DateTime(date('Y-m-d H:i:s')), true);

		    $date_diff = $diff->i + $diff->h * 60 + $diff->d * 60 * 24 + $diff->m * 60 * 24 * 30;
		}
		else $date_diff = 31;

		$xml_article->appendChild(self::addNode('date_diff', $date_diff, $doc));

		$root->appendChild($xml_article);

		$node = $fakedoc->createElement("xtariqa:result");
		$node->setAttribute("id", '_' . $document->id);
		$node->setAttribute("title", $document->title);
		$node->setAttribute("date", date('d/m/Y H:i:s', $document->date));
		$node->setAttribute("publisher", $document->publisher);
		$node->nodeValue = $result->engine . ':' . $result->source;

		$result_ids->appendChild($node);
	    }

	    $html_ids->appendChild($result_ids);

	    $fakedoc->appendChild($html_ids);

	    $node = $doc->createElement("html_ids");
	    $value = $doc->createCDATASection($fakedoc->saveXML());
	    $node->appendChild($value);
	    $root->appendChild($node);

	    if ($result->more_results != null){
		$translations = \tariqa3\statics::getTranslations('search');
		$tpl = new \MetaNull\Application\MTemplate('m');
		$path = TARIQA3_REL_PATH;
		if (substr($path, -1) != '/'){
		    $path .= '/';
		}
		$tpl->loadTemplate($path . 'themes/default/modules/tariqa3/Search/more_results.html');
		$tpl->data = $translations;
		$tpl->data['url'] = $result->more_results;

		$root->appendChild(self::addNode('more_results', $tpl->transform(), $doc, true));
	    }

	    $root->appendChild(self::addNode('spell_check', $result->spell_check, $doc, true));

	    $doc->appendChild($root);
	}
	else if ($result instanceof tariqaFullDocument) {
	    /* @var $result tariqaFullDocument */
	    $root = $doc->createElement('article');
	    $root->appendChild(self::addNode('engine', $result->engine, $doc));
	    $root->appendChild(self::addNode('source', $result->source, $doc));
	    $root->appendChild(self::addNode('id', $result->id, $doc, true));
	    $root->appendChild(self::addNode('document_id', $result->document_id, $doc, true));
	    $root->appendChild(self::addNode('title', $result->title, $doc, true));
	    if ($result->date == 0) {
		if (array_key_exists('search', $newsConfig)) {
		    $d = $newsConfig['search']['dates']['no-date'];
		}
		else {
		    $d = $newsConfig['dates']['no-date'];
		}
	    }
	    else {
		if ($date_format != null) $d = date($date_format, $result->date);
	    }
        
	    $root->appendChild(self::addNode('date', $d, $doc));
	    $root->appendChild(self::addNode('publisher', $result->publisher, $doc, true));
	    $root->appendChild(self::addNode('body', $result->body, $doc, true));
	    $root->appendChild(self::addNode('extra_headers', $result->html_headers, $doc, true));
	    $root->appendChild(self::addNode('additional_controls', ($result->additional_controls != null?$result->additional_controls:''), $doc, true));
        $root->appendChild(self::addNode('additional_main_controls', ($result->additional_main_controls != null?
                                                                      self::controls2string($result->additional_main_controls):''), $doc, true));
	    $root->appendChild(self::addNode('additional_js', ($result->js_code != null?$result->js_code:''), $doc, true));
	    $root->appendChild(self::addNode('additional_jsonload', ($result->js_onload != null?$result->js_onload:''), $doc, true));
	    $root->appendChild(self::addNode('template', ($result->template != null?$result->template:''), $doc, false));

	    $doc->appendChild($root);
	}

	return $doc;
    }

    /**
     * Generates a link for an article
     * @param string $id The article identifier
     * @param string $extra_arguments List of extra arguments GET encoded [OPTIONAL]
     * @access protected
     * @return string
     */
    protected function link($id, $extra_arguments = null) {
	return \tariqa3\statics::URL('index.php', 'module=article&type=new&id=' . $id . ($extra_arguments != null?'&' . $extra_arguments:''));
    }

    /**
     * Generates a javascript link for opening an article
     * @param string $id The article identifier
     * @access protected
     * @return string
     */
    protected function js_link($id, $title) {
	return "openArticle('{$id}', '{$title}', 'new')";
    }

    /**
     * Converts a date from UNIX timestamps to range of days from the current day
     * @param int $date The date
     * @access protected
     * @return int
     */
    protected function date2range($date) {
	return floor((time() - $date) / (60 * 60 * 24)) + 1;
    }

    /**
     * Fetches a list of articles
     * @param newsQuery $query The query
     * @access public
     * @abstract
     * @return tariqaSearchResultSet
     */
    public abstract function listArticles($query);

    /**
     * Gets a specific article
     * @param string $article_id The article identifier
     * @access public
     * @abstract
     * @return DOMDocument
     */
    public abstract function getArticle($article_id);
    
    /**
     * Gets the multi ajax search object
     * @param string $query The query string
     * @param int $from The first result to display
     * @param int $count The number of results to display
     * @param DateTime $startDate The start date [OPTIONAL]
     * @param DateTime $endDate The end date [OPTIONAL]
     * @access public
     * @abstract
     * @return multiAjaxSearch
     */
    public abstract function multiAjaxSearch($query, $from, $count, $startDate = null, $endDate = null);

    /**
     * Gets the POST data used for saving a search in the briefcase
     * @param string $query_txt The query string text
     * @param Query $query The query
     * @param int $start_date The start date (unix timestamp) [OPTIONAL]
     * @param int $end_date The end date (unix timestamp) [OPTIONAL]
     * @param array $additional_parameters Additional parameters [OPTIONAL]
     * @access public
     * @abstract
     * @return string
     */
    public abstract function getSavePost($query_txt, $query, $start_date = null, $end_date = null, $additional_parameters = array());
}
}
?>