<?php
/*
 * © EUROPEAN COMMISSION, 2007-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou implicites.
 * Consultez la Licence pour les autorisations et les restrictions linguistiques spécifiques relevant de la Licence.
 *
 * Auteurs de l'oeuvre:
 *   Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 *   Cosmin POPESCU, Adrian.POPESCU@ext.ec.europa.eu
 *
 */

	/**
	 * Helper functions & classes for LDAP access 
	 * @author Pascal Havelange
 	 * @copyright © EUROPEAN COMMISSION, 2007
 	 * @package MetaNull
 	 */


	/**
	 * LDAP Helper Class
	 * @author Pascal Havelange
 	 * @copyright © EUROPEAN COMMISSION, 2007
 	 * @package MetaNull
	 */
	class mldap {
		
		/**
		 * Resource link identifier
		 * @var resource link identifier
		 */
		protected $hldap;
		
		/**
		 * LDAP Server
		 * @var string
		 */
		public $server;
		
		/**
		 * LDAP Server's port
		 * @var integer
		 */
		public $port;
		
		/**
		 * Limits the quantity of results returned
		 * @var integer
		 * */
		public $max_results = 0;
		
		/**
		 * Limits the duration of the search operation (in seconds)
		 * @var integer
		 * */
		public $max_time = 0;
		
		/**
		 * Sets the dereferencing mechanism
		 * Please use either LDAP_DEREF_NEVER, LDAP_DEREF_SEARCHING, LDAP_DEREF_FINDING, LDAP_DEREF_ALWAYS
		 * @see ldap_search
		 * @var integer
		 * */
		public $deref = LDAP_DEREF_NEVER;
		
		/**
		 * Initialize the mldap object
		 *
		 * @param string $server LDAP Server host
		 * @param int $port LDAP Server's port 
		 * @return mldap
		 */
		public function __construct( $server, $port = 389  ) {
			$this->server = $server;
			$this->port = $port;
			$this->hldap = false;
		}
		
		/**
		 * Connect to the server
		 *
		 * @param string $username The username to use to connect to the LDAP server, or false for anonymous connection
		 * @param string $password The password to use to connect to the LDAP server, or false for anonymous connection
		 * @return boolean True on success, false otherwise
		 */
		public function connect( $username = false, $password = false ) {
			$this->hldap = ldap_connect( $this->server, $this->port );
			if( false === $this->hldap ) {
				return false;
			}
			if( false === $username ) {
				if( false === ldap_bind( $this->hldap )) {
					return false;
				}
			} else {
				if( false === ldap_bind( $this->hldap, $username, $password )) {
					return false;
				}
			}
			return true;
		}
		
		/**
		 * Close the connection with the server
		 *
		 */
		public function close() {
			if( false !== $this->hldap ) {
				ldap_close( $this->hldap );
				$this->hldap = false;
			}
		}
		
		/**
		 * Select a different LDAP server than the one set by the constructor (implicitelly closes the current connection)
		 *
		 * @param string $server LDAP Server host
		 * @param int $port LDAP Server's port
		 */
		public function setHost( $server, $port = 389 ) {
			$this->close();
			$this->server = $server;
			$this->port = $port;
		}
		
		/**
		 * Execute a Search operation and return its results in an array
		 *
		 * @param string $basedn Base DN
		 * @param string $filter Filter
		 * @param array $attributes An array of attribute names to retreive (this array should be numerically indexed)
		 * @return mixed An array of entries on success, or false on error.
		 */
		public function search( $basedn, $filter, $attributes ) {
			if( false === $this->hldap ) {
				return false;
			}
			
			$ldaprc = ldap_search( $this->hldap, $basedn, $filter, $attributes, 0, $this->max_results, $this->max_time, $this->deref );
			if( false === $ldaprc ) {
				return false;
			}
			
			$ldap_entries = ldap_get_entries( $this->hldap, $ldaprc );
			if( false === $ldap_entries ) {
				return false;
			}
		
			$entries = array();
			for( $i = 0; $i < $ldap_entries["count"]; $i ++ ) {
				$entry = $this->entryToArray( $ldap_entries[$i] );
				$entries[] = $entry;
			}
			return $entries;
		}
		
		/**
		 * Get the last LDAP error message
		 *
		 * @return string
		 */
		public function error() {
			if( false === $this->hldap ) {
				return false;
			}
			return ldap_error( $this->hldap );
		}
		
		/**
		 * Get the last LDAP error code
		 *
		 * @return integer
		 */
		public function errno() {
			if( false === $this->hldap ) {
				return false;
			}
			return ldap_errno( $this->hldap );
		}
		
		/**
		 * "Converts" an LDAP attributes array into a classical PHP array
		 * @static 
		 * @param array $ldap_entry an entry returned by a call to ldap_search
		 * @return a PHP array simplifying the ldap entry
		 */
		static function entryToArray( $ldap_entry ) {
			$ldap_attributes = array();
			foreach( $ldap_entry as $entry => $values ) {
				if( "count" == $entry ) {
					continue;
				}
				if( is_array( $values )) {
					if( 2 < count( $values )) {
						$ldap_attributes[$entry] = array();
						foreach( $values as $vidx => $value ) {
							if( "count" == $vidx ) {
								continue;
							}
							$ldap_attributes[$entry][] = $value;
						}
					} else {
						$ldap_attributes[$entry] = $values[0];
					}
				} else {
					// Nothing to do: if not an array, it is a list of attribute names, that wecan find also in the array...
					//$user->ldap_entry[$entry] = $values;
				}
			}
			$ldap_attributes["dn"] = $ldap_entry["dn"];
			return $ldap_attributes;
		}
		
		/**
		 * Test if an attribute exists in the LDAP entry
		 * @static
		 * @param array $ldap_entry The LDAP entry, as returned by @see search or @see entryToArray
		 * @param string $attribute The name of the attribute to search for
		 * @return boolean
		 */
		static function hasAttribute( $ldap_entry, $attribute ) {
			return array_key_exists( $attribute, $ldap_entry ) ? true : false;
		}
		
		/**
		 * Test if an attribute exists in the LDAP entry, and if it haves a given value
		 * @static
		 * @param array $ldap_entry The LDAP entry, as returned by @see search or @see entryToArray
		 * @param string $attribute The name of the attribute to search for
		 * @param string $value The value amongst which to test the attributes
		 * @return boolean
		 */
		static function testAttribute( $ldap_entry, $attribute, $value ) {
			if( !array_key_exists( $attribute, $ldap_entry )) {
				return false;
			}
			return in_array( $value, $ldap_entry[$attribute] ) ? true : false;
		}
		
		/**
		 * Test if an attribute exists in the LDAP entry (Case Insensitive!)
		 * @static
		 * @param array $ldap_entry The LDAP entry, as returned by @see search or @see entryToArray
		 * @param string $attribute The name of the attribute to search for
		 * @return boolean
		 */
		static function hasAttributeI( $ldap_entry, $attribute ) {
			return in_array( strtolower($attribute), array_map( "strtolower", array_keys( $ldap_entry ))) ? true : false;
		}
		
		/**
		 * Test if an attribute exists in the LDAP entry, and if it haves a given value (Case Insensitive!)
		 * @static
		 * @param array $ldap_entry The LDAP entry, as returned by @see search or @see entryToArray
		 * @param string $attribute The name of the attribute to search for
		 * @param string $value The value amongst which to test the attributes
		 * @return boolean
		 */
		static function testAttributeI( $ldap_entry, $attribute, $value ) {
			$keys = array_keys( $ldap_entry );
			$keysi = array_map( "strtolower", $keys );
			
			foreach( $keysi as $idx => $key ) {
				if( $key == $attribute ) {
					$values = array_map( "strtolower", $ldap_entry[$keys[$idx]] );
					return in_array( strtolower($value), $values ) ? true : false;
				}
			}
			return false;
		}
		
	}
?>